#
# common.py
#
# Copyright (C) 2008 Marcos Pinto ('markybob') <markybob@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

"""Common functions for various parts of gtkui to use."""

import pygtk
pygtk.require('2.0')
import gtk, gtk.glade

import pkg_resources

from deluge.ui.client import aclient as client
import deluge.component as component
from deluge.log import LOG as log
import deluge.common

def get_logo(size):
    """Returns a deluge logo pixbuf based on the size parameter."""
    if deluge.common.windows_check():
        return gtk.gdk.pixbuf_new_from_file_at_size(deluge.common.get_pixmap("deluge.png"), \
            size, size)
    else:
        return gtk.gdk.pixbuf_new_from_file_at_size(deluge.common.get_pixmap("deluge.svg"), \
            size, size)

def build_menu_radio_list(value_list, callback, pref_value=None,
    suffix=None, show_notset=False, notset_label=None, notset_lessthan=0,
    show_other=False, show_activated=False, activated_label=None):
    # Build a menu with radio menu items from a list and connect them to
    # the callback. The pref_value is what you would like to test for the
    # default active radio item.
    if notset_label is None:
        notset_label = _("Unlimited")

    if activated_label is None:
        activated_label = _("Activated")

    menu = gtk.Menu()
    group = None
    if show_activated is False:
        if pref_value > -1 and pref_value not in value_list:
            value_list.pop()
            value_list.append(pref_value)

        for value in sorted(value_list):
            if suffix != None:
                menuitem = gtk.RadioMenuItem(group, str(value) + " " + \
                    suffix)
            else:
                menuitem = gtk.RadioMenuItem(group, str(value))

            group = menuitem

            if value == pref_value and pref_value != None:
                menuitem.set_active(True)

            if callback != None:
                menuitem.connect("toggled", callback)

            menu.append(menuitem)

    if show_activated is True:
        for value in sorted(value_list):
            menuitem = gtk.RadioMenuItem(group, str(activated_label))

            group = menuitem

            if value == pref_value and pref_value != None:
                menuitem.set_active(True)

            if callback != None:
                menuitem.connect("toggled", callback)

            menu.append(menuitem)

    if show_notset:
        menuitem = gtk.RadioMenuItem(group, notset_label)
        menuitem.set_name(notset_label)
        if pref_value < notset_lessthan and pref_value != None:
            menuitem.set_active(True)
        if show_activated and pref_value == 1:
            menuitem.set_active(True)
        menuitem.connect("toggled", callback)
        menu.append(menuitem)

    # Add the Other... menuitem
    if show_other is True:
        menuitem = gtk.SeparatorMenuItem()
        menu.append(menuitem)
        menuitem = gtk.MenuItem(_("Other..."))
        menuitem.set_name(_("Other..."))
        menuitem.connect("activate", callback)
        menu.append(menuitem)

    return menu

def show_other_dialog(header, type_str, image_stockid=None, image_filename=None, default=0):
    """
    Shows a dialog with :param:`header` as the header text and :param:`type_str`
    as the type text.  The type of spinbutton (int or float) is determined by
    :param:`default`s type.

    :param header: str, the header label text
    :param type_str: str, the type label text, what comes after the spinbutton
    :param image_stockid: gtkStockId, the stock id of the image in the header
    :param image_filename: str, filename of icon in pixmaps folder
    :param default: the default value in the spinbutton

    :returns: None, int or float from spinbutton depending on :param:`default`.
        None is returned if the user clicks on Cancel.
    :rtype: None, int or float

    :raises TypeError: if :param:`default` is not of type int or float

    """
    if type(default) != int and type(default) != float:
        raise TypeError("default value needs to be an int or float")

    glade = gtk.glade.XML(
        pkg_resources.resource_filename("deluge.ui.gtkui",
                                    "glade/dgtkpopups.glade"))
    dialog = glade.get_widget("other_dialog")
    dialog.set_transient_for(component.get("MainWindow").window)
    dialog.set_title("")
    glade.get_widget("label_header").set_markup("<b>" + header + "</b>")
    glade.get_widget("label_type").set_text(type_str)
    if image_stockid:
        glade.get_widget("image").set_from_stock(image_stockid, gtk.ICON_SIZE_LARGE_TOOLBAR)
    if image_filename:
        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
            deluge.common.get_pixmap(image_filename), 32, 32)
        glade.get_widget("image").set_from_pixbuf(pixbuf)

    spinbutton = glade.get_widget("spinbutton")
    if type(default) == float:
        spinbutton.set_digits(1)

    # Set default value and select text
    spinbutton.set_value(default)
    spinbutton.select_region(0, -1)

    value = None
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        if type(default) == int:
            value = spinbutton.get_value_as_int()
        else:
            value = spinbutton.get_value()

    dialog.destroy()
    return value

def add_peer_dialog():
    dialog_glade = gtk.glade.XML(
        pkg_resources.resource_filename("deluge.ui.gtkui",
            "glade/dgtkpopups.glade"))
    peer_dialog = dialog_glade.get_widget("connect_peer_dialog")
    txt_ip = dialog_glade.get_widget("txt_ip")
    response = peer_dialog.run()
    if response:
        value = txt_ip.get_text()
        if ']' in value:
            #ipv6
            ip = value.split("]")[0][1:]
            port = value.split("]")[1][1:]
        else:
            #ipv4
            ip = value.split(":")[0]
            port = value.split(":")[1]
        if deluge.common.is_ip(ip):
            id = component.get("TorrentView").get_selected_torrent()
            log.debug("adding peer %s to %s", value, id)
            client.connect_peer(id, ip, port)
    peer_dialog.destroy()
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

#


#todo: remove useless imports.
import base64

from utils import *
import utils #todo remove the line above.
from render import render, error_page
import page_decorators as deco

from deluge.common import get_pixmap
from deluge.log import LOG as log

import web
from web import url
from lib.static_handler import static_handler

from operator import attrgetter
import os
from deluge import component
from deluge.ui.client import sclient as proxy
from deluge.configmanager import ConfigManager
from deluge.ui.tracker_icons import TrackerIcons

page_manager = component.get("PageManager")
config = ConfigManager("webui06.conf")

def route(url, klass, use_module=True):
    """
    url-mapping is using page_manager
    not the default web.py way ; I want class decorators!
    """
    page_manager.register_page(url, klass,use_module)

#pages:
class login:
    @deco.deluge_page_noauth
    def GET(self, name):
        vars = web.input(error = None)
        return render.login(vars.error)

    def POST(self):
        vars = web.input(pwd = None, redir = None)

        if utils.check_pwd(vars.pwd):
            #start new session
            start_session()
            utils.seeother('/index')
        elif vars.redir:
            utils.seeother(url('/login', error=1, redir=vars.redir))
        else:
            utils.seeother('/login?error=1')

route('/login',login)

class index:
    "page containing the torrent list."
    @deco.deluge_page
    @deco.auto_refreshed
    def GET(self, name):
        vars = web.input(sort=None, order=None, filter_cat=None ,filter_value=None , tracker=None)

        if not vars.sort: #no arguments, default to coockies.
            newvars = cookies()
            if vars.filter_cat:
                newvars['filter_cat'] = vars.filter_cat
                newvars['filter_value'] = vars.filter_value
            vars.update(newvars)

        else: #has arguments:set cookies from arguments.
            for key in ["sort", "order", "filter_cat","filter_value"]:
                value = getattr(vars, key) or ""
                setcookie(key, value)

        #Filters
        filter_dict = {}
        if vars.filter_cat and vars.filter_value <> "All":
            filter_dict = {vars.filter_cat:vars.filter_value}

        torrents =  proxy.get_torrents_status(filter_dict, TORRENT_KEYS)

        #sidebar-config:
        hide_cat = []
        if not config["sidebar_show_trackers"]:
            hide_cat = ["tracker_host"]

        if config["show_sidebar"]:
            label_filters = proxy.get_filter_tree(config["sidebar_show_zero"], hide_cat)
        else:
            label_filters = {}

        torrent_list = utils.get_enhanced_torrent_list(torrents)

        #sorting:
        if vars.sort:
            try:
                torrent_list.sort(key=attrgetter(vars.sort))
            except:
                log.error('Sorting Failed')

            if vars.order == 'up':
                torrent_list = list(reversed(torrent_list))

        return render.index(torrent_list, label_filters)
route('/index',index)

#simple proxy's to deluge.ui.client
#execute a command on torrent(s) and redirect to index page.
def reg_torrents_POST(url_name, proxy_command):
    class _page_class:
        @deco.check_session
        @deco.torrent_ids
        def POST(self, torrent_ids):
            getattr(proxy, proxy_command)(torrent_ids)
            do_redirect()
    _page_class.__name__ = "TORRENTS_POST:" + proxy_command
    route("/torrent/%s/(.*)" % url_name, _page_class, use_module=False)

reg_torrents_POST("start", "resume_torrent")
reg_torrents_POST("stop", "pause_torrent")
reg_torrents_POST("recheck", "force_recheck")
reg_torrents_POST("reannounce", "force_reannounce")
reg_torrents_POST("queue/up", "queue_up")
reg_torrents_POST("queue/down", "queue_down")
reg_torrents_POST("queue/top", "queue_top")
reg_torrents_POST("queue/bottom", "queue_bottom")


class torrent_info:
    @deco.deluge_page
    @deco.auto_refreshed
    @deco.torrent
    def GET(self, torrent):
        return render.torrent_info(torrent)
route("/torrent/info/(.*)", torrent_info)

class torrent_info_inner:
    @deco.deluge_page
    @deco.torrent
    def GET(self, torrent):
        vars = web.input(tab = None)
        if vars.tab:
            active_tab = vars.tab
        else:
            active_tab =  getcookie("torrent_info_tab") or "details"
        setcookie("torrent_info_tab", active_tab)
        return render.torrent_info_inner(torrent, active_tab)
route("/torrent/info_inner/(.*)", torrent_info_inner)

class torrent_delete:
    @deco.deluge_page
    @deco.torrent_list
    def GET(self, torrent_list):
            torrent_str = ",".join([t.id for t in torrent_list])
            #todo: remove the ",".join!
            return render.torrent_delete(torrent_str, torrent_list)

    @deco.check_session
    @deco.torrent_ids
    def POST(self, torrent_ids):
        vars = web.input(data_also = None, torrent_also = None)
        data_also = bool(vars.data_also)
        torrent_also = bool(vars.torrent_also)
        proxy.remove_torrent(torrent_ids, data_also)
        do_redirect()
route("/torrent/delete/(.*)",torrent_delete)

class torrent_files:
    @deco.deluge_page
    @deco.torrent
    def GET(self,torrent):
        return render.torrent_files(torrent, None)

    @deco.check_session
    @deco.torrent
    def POST(self, torrent):
        values = web.input()
        file_priorities = []
        for i in xrange(len(torrent.files)):
            file_priorities.append(int(getattr(values,"prio_%s" % i )))

        proxy.set_torrent_file_priorities(torrent.id, file_priorities)
        do_redirect()
route("/torrent/files/(.*)", torrent_files)

class torrent_trackers:
    @deco.check_session
    @deco.torrent
    def POST(self, torrent):
        vars = web.input(tier=[], url=[])

        tiers_int = [int(t) for t in vars.tier]
        sorted_urls =  [url for num,url in sorted(zip(tiers_int, vars.url)) if url]
        trackers = [{'tier':i , 'url':url} for i,url in enumerate(sorted_urls)]

        proxy.set_torrent_trackers(torrent.id, trackers)
        do_redirect()
route("/torrent/trackers/(.*)", torrent_trackers)

class pause_all:
    @deco.check_session
    def POST(self, name):
        proxy.pause_torrent(proxy.get_session_state())
        do_redirect()
route("/pause_all", pause_all)

class resume_all:
    @deco.check_session
    def POST(self, name):
        proxy.resume_torrent(proxy.get_session_state())
        do_redirect()
route("/resume_all", resume_all)

class refresh:
    def GET(self, name):
        return self.POST(name)
        #WRONG, but makes it easyer to link with <a href> in the status-bar

    @deco.check_session
    def POST(self, name):
        auto_refresh = {'off': '0', 'on': '1'}[name]
        setcookie('auto_refresh', auto_refresh)
        if not getcookie('auto_refresh_secs'):
            setcookie('auto_refresh_secs', 10)
        do_redirect()
route("/refresh/(.*)", refresh)

class refresh_set:
    @deco.deluge_page
    def GET(self, name):
        return render.refresh_form()

    @deco.check_session
    def POST(self, name):
        vars = web.input(refresh = 0)
        refresh = int(vars.refresh)
        if refresh > 0:
            setcookie('auto_refresh', '1')
            setcookie('auto_refresh_secs', str(refresh))
            do_redirect()
        else:
            error_page(_('refresh must be > 0'))
route("/refresh/set", refresh_set)

class home:
    @deco.check_session
    def GET(self, name):
        do_redirect()
route('/home', home)
route('/', home)

class about:
    @deco.deluge_page_noauth
    def GET(self, name):
        return render.about()
route('/about', about)

class logout:
    def GET(self):
        return self.POST()
        #WRONG, but makes it easyer to link with <a href> in the status-bar
    @deco.check_session
    def POST(self, name):
        end_session()
        utils.seeother('/login')
route('/logout', logout)

class connect:
    @deco.check_session
    @deco.deluge_page_noauth
    def GET(self, name):
        restart = False
        try:
            proxy.ping()
            connected = proxy.get_core_uri()
            if connected.startswith("http://localhost"):
                restart = True
        except:
            connected = None

        connect_list = ["http://localhost:58846"]
        if config['daemon'] <> "http://localhost:58846":
            connect_list = [config['daemon']] + connect_list

        return render.connect(connect_list, connected ,restart)

    def POST(self):
        vars = web.input(uri = None, other_uri = None)
        uri = ''
        print "URI=",uri
        if vars.uri == 'other':
            if not vars.other_uri:
                return error_page(_("no uri"))
            uri = vars.other_uri
        else:
            uri = vars.uri
        #TODO: more error-handling
        utils.daemon_connect(uri)
        do_redirect()
route('/connect', connect)

class daemon_control:
    @deco.check_session
    def POST(self, command):
        if command == 'stop':
            proxy.shutdown()
        elif command == 'start':
            self.start()
            return do_redirect()
        elif command == 'restart':
            proxy.shutdown()
            self.start()
            return do_redirect()
        else:
            raise Exception('Unknown command:"%s"' % command)

        utils.seeother('/connect')

    def start(self):
        import time
        uri = web.input(uri = None).uri
        if not uri:
            uri = 'http://localhost:58846'
        port = int(uri.split(':')[2])
        utils.daemon_start_localhost(port)
        time.sleep(1)  #pause a while to let it start?
        utils.daemon_connect( uri )
route("/daemon/control/(.*)", daemon_control)

#other stuff:
class remote_torrent_add:
    """
    For use in remote scripts etc.
    curl ->POST pwd and torrent as file
    greasemonkey: POST pwd torrent_name and data_b64
    """
    @deco.remote
    def POST(self, name):
        vars = web.input(pwd = None, torrent = {},
            data_b64 = None , torrent_name= None)

        if not utils.check_pwd(vars.pwd):
            return 'error:wrong password'

        if vars.data_b64: #b64 post (greasemonkey)
            data_b64 = base64.decodestring(unicode(vars.data_b64))
            torrent_name = vars.torrent_name
        else:  #file-post (curl)
            data_b64 = base64.b64encode(vars.torrent.file.read())
            torrent_name = vars.torrent.filename
        proxy.add_torrent_file_binary(torrent_name, data_b64)
        return 'ok'
route("/remote/torrent/add(.*)", remote_torrent_add)

class static(static_handler):
    base_dir = os.path.join(os.path.dirname(__file__), 'static')
route("/static/(.*)", static)

class template_static(static_handler):
    def get_base_dir(self):
        return os.path.join(os.path.dirname(__file__),
                'templates/%s/static' % config['template'])
route("/template/static/(.*)", template_static)

class template_render:
    "render anything in /render/ dir"

    def GET(self, name):
        web.header("Content-type",utils.guess_mime_type(name))
        #security : assumes config['template'] returns a safe subdir.
        basepath = os.path.normpath(os.path.join(os.path.dirname(__file__),
                'templates/%s/render' % config['template']))
        filename =  os.path.normpath(os.path.join(basepath,name))
        if not filename.startswith(basepath):
            #hack detected?
            raise Exception("File to render is not located in %s" % basepath)

        print web.template.Template(open(filename).read(), filename=filename)()
route("/template/render/(.*)", template_render)

class template_style:
    def GET(self):
        web.header("Content-Type", "text/css")
        style = Storage()
        print render.template_style(style)
route("/template_style.css", template_style)

class robots:
    def GET(self):
        "no robots/prevent searchengines from indexing"
        web.header("Content-Type", "text/plain")
        print "User-agent: *\nDisallow:/\n"
route("/robots.txt", robots)

class pixmaps:
    "use the deluge-images. located in data/pixmaps"
    def GET(self, name):
        if name.startswith("flags") and not name.endswith('.png'):
            name = name + ".png"
        if not name.endswith('.png'):
            if name == 'paused':
                name = 'inactive'
            if name == 'error':
                name = 'alert'
            name =  name + '16.png'

        if not os.path.exists(get_pixmap(name)):
            name = 'dht16.png'

        f = open(get_pixmap(name) ,'rb')
        fs = os.fstat(f.fileno())
        content = f.read()
        f.close()
        web.header("Content-Type", "image/png")
        web.header("Content-Length", str(fs[6]))
        web.header("Cache-Control" , "public, must-revalidate, max-age=86400")
        print content
route("/pixmaps/(.*)", pixmaps)


class tracker_icon:
    tracker_icons = TrackerIcons()
    def GET(self, name):
        filename = self.tracker_icons.get(name)
        if filename:
            log.debug("file-name=%s" %  name)
            web.header("Cache-Control" , "public, must-revalidate, max-age=86400")
            if filename.endswith(".ico"):
                web.header("Content-Type", "image/x-icon")
            elif filename.endswith(".png"):
                web.header("Content-Type", "image/png")
            data = open(filename, "rb").read()
            print data
        else:
            log.debug("not found:%s" % name)
            web.header("Content-Type", "image/x-icon")

route("/tracker/icon/(.*)", tracker_icon)

class close:
    "close open window"
    @deco.deluge_page
    def GET(self, name):
        return """
        <script language="javascript">
        window.close();
        </script>
        """
route("/close", close)

class gettext:
    """"
    gettext.js
    """
    def GET(self):
        web.header("Content-Type", "text/javascript")
        print render.gettext()
route("/gettext.js", gettext)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

#

"""
this is an ugly hack, and it will be changed.
for experimental use only!!
"""
import os

def get_wsgi_application(base_url, config_dir):

    #monkeypatch:
    from deluge import common
    def get_config_dir(filename = ""):
        return os.path.join(config_dir, filename)
    common.get_config_dir = get_config_dir
    #/monkeypatch

    from deluge.configmanager import ConfigManager
    from deluge.ui.webui import deluge_webserver
    from deluge.ui.webui import utils

    config = ConfigManager("webui06.conf")

    utils.set_config_defaults()

    config['base'] = '/deluge'
    config['disallow'] = {
        '/daemon/control':'running as an apache user',
        '/config/server':'running as an apache-user'
        }

    utils.apply_config()

    return deluge_webserver.WsgiApplication()



#
# log.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


"""Logging functions"""

import logging

levels = {
    "info": logging.INFO,
    "warning": logging.WARNING,
    "error": logging.ERROR,
    "none": logging.CRITICAL,
    "debug": logging.DEBUG
}
def setupLogger(level="error", filename=None):
    """
    Sets up the basic logger and if `:param:filename` is set, then it will log
    to that file instead of stdout.

    :param level: str, the level to log
    :param filename: str, the file to log to
    """

    if not level or level not in levels:
        level = "error"

    logging.basicConfig(
        level=levels[level],
        format="[%(levelname)-8s] %(asctime)s %(module)s:%(lineno)d %(message)s",
        datefmt="%H:%M:%S",
        filename=filename,
        filemode="w"
    )

def setLoggerLevel(level):
    """
    Sets the logger level.

    :param level: str, a string representing the desired level

    """
    if level not in levels:
        return

    global LOG
    LOG.setLevel(levels[level])

# Get the logger
LOG = logging.getLogger("deluge")



#
# pluginmanagerbase.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


"""PluginManagerBase"""

import os.path

import pkg_resources

import deluge.common
import deluge.configmanager
from deluge.log import LOG as log

METADATA_KEYS = [
    "Name",
    "License",
    "Author",
    "Home-page",
    "Summary",
    "Platform",
    "Version",
    "Author-email",
    "Description",
]

class PluginManagerBase:
    """PluginManagerBase is a base class for PluginManagers to inherit"""

    def __init__(self, config_file, entry_name):
        log.debug("Plugin manager init..")

        self.config = deluge.configmanager.ConfigManager(config_file)

        # Create the plugins folder if it doesn't exist
        if not os.path.exists(os.path.join(deluge.configmanager.get_config_dir(), "plugins")):
            os.mkdir(os.path.join(deluge.configmanager.get_config_dir(), "plugins"))

        # This is the entry we want to load..
        self.entry_name = entry_name

        # Loaded plugins
        self.plugins = {}

        # Scan the plugin folders for plugins
        self.scan_for_plugins()

    def enable_plugins(self):
        # Load plugins that are enabled in the config.
        for name in self.config["enabled_plugins"]:
            self.enable_plugin(name)

    def disable_plugins(self):
        # Disable all plugins that are enabled
        for key in self.plugins.keys():
            self.plugins[key].disable()

    def __getitem__(self, key):
        return self.plugins[key]

    def get_available_plugins(self):
        """Returns a list of the available plugins name"""
        return self.available_plugins

    def get_enabled_plugins(self):
        """Returns a list of enabled plugins"""
        return self.plugins.keys()

    def scan_for_plugins(self):
        """Scans for available plugins"""
        plugin_dir = os.path.join(os.path.dirname(__file__), "plugins")
        user_plugin_dir = os.path.join(deluge.configmanager.get_config_dir(), "plugins")

        pkg_resources.working_set.add_entry(plugin_dir)
        pkg_resources.working_set.add_entry(user_plugin_dir)
        self.pkg_env = pkg_resources.Environment([plugin_dir, user_plugin_dir])

        self.available_plugins = []
        for name in self.pkg_env:
            log.debug("Found plugin: %s %s",
                self.pkg_env[name][0].project_name,
                self.pkg_env[name][0].version)
            self.available_plugins.append(self.pkg_env[name][0].project_name)

    def enable_plugin(self, plugin_name):
        """Enables a plugin"""
        if plugin_name not in self.available_plugins:
            log.warning("Cannot enable non-existant plugin %s", plugin_name)
            return

        plugin_name = plugin_name.replace(" ", "-")
        egg = self.pkg_env[plugin_name][0]
        egg.activate()
        for name in egg.get_entry_map(self.entry_name):
            entry_point = egg.get_entry_info(self.entry_name, name)
            cls = entry_point.load()
            instance = cls(self, plugin_name.replace("-", "_"))
            instance.enable()
            plugin_name = plugin_name.replace("-", " ")
            self.plugins[plugin_name] = instance
            if plugin_name not in self.config["enabled_plugins"]:
                log.debug("Adding %s to enabled_plugins list in config",
                    plugin_name)
                self.config["enabled_plugins"].append(plugin_name)
            log.info("Plugin %s enabled..", plugin_name)

    def disable_plugin(self, name):
        """Disables a plugin"""
        try:
            self.plugins[name].disable()
            del self.plugins[name]
            self.config["enabled_plugins"].remove(name)
        except KeyError:
            log.warning("Plugin %s is not enabled..", name)

        log.info("Plugin %s disabled..", name)

    def get_plugin_info(self, name):
        """Returns a dictionary of plugin info from the metadata"""
        info = {}.fromkeys(METADATA_KEYS)
        last_header = ""
        cont_lines = []
        for line in self.pkg_env[name][0].get_metadata("PKG-INFO").splitlines():
            if not line:
                continue
            if line[0] in ' \t' and (len(line.split(":", 1)) == 1 or line.split(":", 1)[0] not in info.keys()):
                # This is a continuation
                cont_lines.append(line.strip())
            else:
                if cont_lines:
                    info[last_header] = "\n".join(cont_lines).strip()
                    cont_lines = []
                if line.split(":", 1)[0] in info.keys():
                    last_header = line.split(":", 1)[0]
                    info[last_header] = line.split(":", 1)[1].strip()

        return info



#
# torrentdetails.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


"""The torrent details component shows info about the selected torrent."""

import gtk, gtk.glade
import os
import os.path
import cPickle

import deluge.component as component
from deluge.ui.client import aclient as client
from deluge.configmanager import ConfigManager
import deluge.configmanager

from deluge.log import LOG as log

class Tab:
    def __init__(self):
        self.is_visible = True
        self.position = -1
        self.weight = -1

    def get_name(self):
        return self._name

    def get_child_widget(self):
        parent = self._child_widget.get_parent()
        if parent is not None:
            parent.remove(self._child_widget)

        return self._child_widget

    def get_tab_label(self):
        parent = self._tab_label.get_parent()
        log.debug("parent: %s", parent)
        if parent is not None:
            parent.remove(self._tab_label)

        return self._tab_label

class TorrentDetails(component.Component):
    def __init__(self):
        component.Component.__init__(self, "TorrentDetails", interval=2000)
        self.window = component.get("MainWindow")
        glade = self.window.main_glade

        self.notebook = glade.get_widget("torrent_info")

        # This is the menu item we'll attach the tabs checklist menu to
        self.menu_tabs = glade.get_widget("menu_tabs")

        self.notebook.connect("switch-page", self._on_switch_page)

        # Tabs holds references to the Tab objects by their name
        self.tabs = {}

        # Add the default tabs
        from status_tab import StatusTab
        from details_tab import DetailsTab
        from files_tab import FilesTab
        from peers_tab import PeersTab
        from options_tab import OptionsTab

        default_tabs = {
            "Status": StatusTab,
            "Details": DetailsTab,
            "Files": FilesTab,
            "Peers": PeersTab,
            "Options": OptionsTab
            }

        # tab_name, visible
        default_order = [
            ("Status", True),
            ("Details", True),
            ("Files", True),
            ("Peers", True),
            ("Options", True)
        ]

        # Get the state from saved file
        state = self.load_state()

        if state:
            for item in state:
                if not isinstance(item, tuple):
                    log.debug("Old tabs.state, using default..")
                    state = None
                    break

        # The state is a list of tab_names in the order they should appear
        if state == None:
            # Set the default order
            state = default_order

        # Add the tabs in the order from the state
        for tab_name, visible in state:
            # We need to rename the tab in the state for backwards compat
            tab_name = tab_name.replace("Statistics", "Status")
            self.add_tab(default_tabs[tab_name]())

        # Hide any of the non-visible ones
        for tab_name, visible in state:
            if not visible:
                self.hide_tab(tab_name)

        # Generate the checklist menu
        self.generate_menu()

    def add_tab(self, tab_object, position=-1, generate_menu=True):
        """Adds a tab object to the notebook."""
        self.tabs[tab_object.get_name()] = tab_object
        pos = self.notebook.insert_page(
            tab_object.get_child_widget(),
            tab_object.get_tab_label(),
            position)

        tab_object.position = pos
        tab_object.weight = pos

        # Regenerate positions if an insert occured
        if position > -1:
            self.regenerate_positions()

        if generate_menu:
            self.generate_menu()

        if not self.notebook.get_property("visible"):
            # If the notebook isn't visible, show it
            self.visible(True)

    def regenerate_positions(self):
        """This will sync up the positions in the tab, with the position stored
        in the tab object"""
        for tab in self.tabs:
            page_num = self.notebook.page_num(self.tabs[tab]._child_widget)
            if page_num > -1:
                self.tabs[tab].position = page_num

    def remove_tab(self, tab_name):
        """Removes a tab by name."""
        self.notebook.remove_page(self.tabs[tab_name].position)
        del self.tabs[tab_name]
        self.regenerate_positions()
        self.generate_menu()

        # If there are no tabs visible, then do not show the notebook
        if len(self.tabs) == 0:
            self.visible(False)

    def hide_all_tabs(self):
        """Hides all tabs"""
        log.debug("n_pages: %s", self.notebook.get_n_pages())
        for n in xrange(self.notebook.get_n_pages() - 1, -1, -1):
            self.notebook.remove_page(n)

        for tab in self.tabs:
            self.tabs[tab].is_visible = False
        log.debug("n_pages: %s", self.notebook.get_n_pages())
        self.generate_menu()
        self.visible(False)

    def show_all_tabs(self):
        """Shows all tabs"""
        for tab in self.tabs:
            if not self.tabs[tab].is_visible:
                self.notebook.insert_page(
                    self.tabs[tab].get_child_widget(),
                    self.tabs[tab].get_tab_label(),
                    self.tabs[tab].position)
                self.tabs[tab].is_visible = True
        self.generate_menu()
        if not self.notebook.get_property("visible"):
            # If the notebook isn't visible, show it
            self.visible(True)

    def hide_tab(self, tab_name):
        """Hides tab by name"""
        self.notebook.remove_page(self.tabs[tab_name].position)
        self.tabs[tab_name].is_visible = False
        self.regenerate_positions()
        self.generate_menu()

        show = False
        for name, tab in self.tabs.iteritems():
            show = show or tab.is_visible

        self.visible(show)

    def show_tab(self, tab_name):
        log.debug("%s\n%s\n%s", self.tabs[tab_name].get_child_widget(),
            self.tabs[tab_name].get_tab_label(),
            self.tabs[tab_name].position)

        # Determine insert position based on weight
        # weights is a list of visible tab names in weight order
        weights = []

        for tab in self.tabs:
            if self.tabs[tab].is_visible:
                weights.append((self.tabs[tab].weight, self.tabs[tab].get_name()))

        weights.sort()
        log.debug("weights: %s", weights)
        position = self.tabs[tab_name].position
        log.debug("weight of tab: %s", self.tabs[tab_name].weight)
        for i, w in enumerate(weights):
            if w[0] >= self.tabs[tab_name].weight:
                position = self.tabs[w[1]].position
                break

        log.debug("position: %s", position)
        self.notebook.insert_page(
            self.tabs[tab_name].get_child_widget(),
            self.tabs[tab_name].get_tab_label(),
            position)
        self.tabs[tab_name].is_visible = True
        self.regenerate_positions()
        self.generate_menu()
        self.visible(True)

    def generate_menu(self):
        """Generates the checklist menu for all the tabs and attaches it"""
        menu = gtk.Menu()
        # Create 'All' menuitem and a separator
        menuitem = gtk.CheckMenuItem("All")

        all_tabs = True
        for key in self.tabs:
            if not self.tabs[key].is_visible:
                all_tabs = False
                break
        menuitem.set_active(all_tabs)
        menuitem.connect("toggled", self._on_menuitem_toggled)

        menu.append(menuitem)

        menuitem = gtk.SeparatorMenuItem()
        menu.append(menuitem)

        # Create a list in order of tabs to create menu
        menuitem_list = []
        for tab_name in self.tabs:
            menuitem_list.append((self.tabs[tab_name].weight, tab_name))
        menuitem_list.sort()

        for pos, name in menuitem_list:
            menuitem = gtk.CheckMenuItem(name)
            menuitem.set_active(self.tabs[name].is_visible)
            menuitem.connect("toggled", self._on_menuitem_toggled)
            menu.append(menuitem)

        self.menu_tabs.set_submenu(menu)
        self.menu_tabs.show_all()

    def visible(self, visible):
        if visible:
            self.notebook.show()
        else:
            self.notebook.hide()
            self.window.vpaned.set_position(-1)

    def set_tab_visible(self, tab_name, visible):
        """Sets the tab to visible"""
        log.debug("set_tab_visible name: %s visible: %s", tab_name, visible)
        if visible and not self.tabs[tab_name].is_visible:
            self.show_tab(tab_name)
        elif not visible and self.tabs[tab_name].is_visible:
            self.hide_tab(tab_name)

    def start(self):
        for tab in self.tabs.values():
            try:
                tab.start()
            except AttributeError:
                pass

    def stop(self):
        self.clear()
        for tab in self.tabs.values():
            try:
                tab.stop()
            except AttributeError:
                pass


    def shutdown(self):
        # Save the state of the tabs
        for tab in self.tabs:
            try:
                self.tabs[tab].save_state()
            except AttributeError:
                pass

        # Save tabs state
        self.save_state()

    def update(self, page_num=None):
        if len(component.get("TorrentView").get_selected_torrents()) == 0:
            # No torrents selected, so just clear
            self.clear()

        if self.notebook.get_property("visible"):
            if page_num == None:
                page_num = self.notebook.get_current_page()
            try:
                # Get the tab name
                name = None
                for tab in self.tabs:
                    if self.tabs[tab].position == page_num and self.tabs[tab].is_visible:
                        name = tab
            except IndexError:
                return
            # Update the tab that is in view
            if name:
                self.tabs[name].update()

    def clear(self):
        # Get the tab name
        try:
            page_num = self.notebook.get_current_page()
            name = None
            for tab in self.tabs:
                if self.tabs[tab].position == page_num and self.tabs[tab].is_visible:
                    name = tab
            if name:
                self.tabs[name].clear()
        except Exception, e:
            log.debug("Unable to clear torrentdetails: %s", e)

    def _on_switch_page(self, notebook, page, page_num):
        self.update(page_num)
        client.force_call(False)

    def _on_menuitem_toggled(self, widget):
        # Get the tab name
        name = widget.get_child().get_text()
        if name == "All":
            if widget.get_active():
                self.show_all_tabs()
            else:
                self.hide_all_tabs()
            return

        self.set_tab_visible(name, widget.get_active())

    def save_state(self):
        """We save the state, which is basically the tab_index list"""
        filename = "tabs.state"
        state = []
        for tab in self.tabs:
            state.append((self.tabs[tab].weight, self.tabs[tab].get_name(),
                self.tabs[tab].is_visible))
        # Sort by weight
        state.sort()
        state = [(n, v) for w, n, v in state]

        # Get the config location for saving the state file
        config_location = deluge.configmanager.get_config_dir()

        try:
            log.debug("Saving TorrentDetails state file: %s", filename)
            state_file = open(os.path.join(config_location, filename), "wb")
            cPickle.dump(state, state_file)
            state_file.close()
        except IOError, e:
            log.warning("Unable to save state file: %s", e)

    def load_state(self):
        filename = "tabs.state"
        # Get the config location for loading the state file
        config_location = deluge.configmanager.get_config_dir()
        state = None

        try:
            log.debug("Loading TorrentDetails state file: %s", filename)
            state_file = open(os.path.join(config_location, filename), "rb")
            state = cPickle.load(state_file)
            state_file.close()
        except (EOFError, IOError), e:
            log.warning("Unable to load state file: %s", e)

        return state



#
# authmanager.py
#
# Copyright (C) 2008 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

import os.path
import random
import stat

import deluge.component as component
import deluge.configmanager as configmanager

class AuthManager(component.Component):
    def __init__(self):
        component.Component.__init__(self, "AuthManager")
        self.auth = {}

    def start(self):
        self.__load_auth_file()

    def stop(self):
        self.auth = {}

    def shutdown(self):
        pass

    def authorize(self, username, password):
        """
        Authorizes users based on username and password

        :param username: str, username
        :param password: str, password
        :returns: True or False
        :rtype: bool

        """

        if username not in self.auth:
            # Let's try to re-load the file.. Maybe it's been updated
            self.__load_auth_file()
            if username not in self.auth:
                return False

        if self.auth[username] == password:
            return True

        return False

    def __load_auth_file(self):
        auth_file = configmanager.get_config_dir("auth")
        # Check for auth file and create if necessary
        if not os.path.exists(auth_file):
            # We create a 'localclient' account with a random password
            try:
                from hashlib import sha1 as sha_hash
            except ImportError:
                from sha import new as sha_hash
            open(auth_file, "w").write("localclient:" + sha_hash(str(random.random())).hexdigest() + "\n")
            # Change the permissions on the file so only this user can read/write it
            os.chmod(auth_file, stat.S_IREAD | stat.S_IWRITE)

        # Load the auth file into a dictionary: {username: password, ...}
        f = open(auth_file, "r")
        for line in f:
            if line.startswith("#"):
                # This is a comment line
                continue
            try:
                username, password = line.split(":")
            except ValueError:
                continue
            self.auth[username.strip()] = password.strip()



#
# toolbar.py
#
# Copyright (C) 2007, 2008 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


import pygtk
pygtk.require('2.0')
import gtk, gtk.glade
import gobject

import deluge.component as component
from deluge.log import LOG as log
from deluge.common import TORRENT_STATE
from deluge.ui.client import aclient as client
from deluge.configmanager import ConfigManager

class ToolBar(component.Component):
    def __init__(self):
        component.Component.__init__(self, "ToolBar")
        log.debug("ToolBar Init..")
        self.window = component.get("MainWindow")
        self.toolbar = self.window.main_glade.get_widget("toolbar")
        self.config = ConfigManager("gtkui.conf")
        ### Connect Signals ###
        self.window.main_glade.signal_autoconnect({
            "on_toolbutton_add_clicked": self.on_toolbutton_add_clicked,
            "on_toolbutton_remove_clicked": self.on_toolbutton_remove_clicked,
            "on_toolbutton_pause_clicked": self.on_toolbutton_pause_clicked,
            "on_toolbutton_resume_clicked": self.on_toolbutton_resume_clicked,
            "on_toolbutton_preferences_clicked": \
                self.on_toolbutton_preferences_clicked,
            "on_toolbutton_connectionmanager_clicked": \
                self.on_toolbutton_connectionmanager_clicked,
            "on_toolbutton_queue_up_clicked": self.on_toolbutton_queue_up_clicked,
            "on_toolbutton_queue_down_clicked": self.on_toolbutton_queue_down_clicked
        })
        self.change_sensitivity = [
            "toolbutton_add",
            "toolbutton_remove",
            "toolbutton_pause",
            "toolbutton_resume",
            "toolbutton_queue_up",
            "toolbutton_queue_down"
        ]

        if self.config["classic_mode"]:
            self.window.main_glade.get_widget("toolbutton_connectionmanager").hide()

        # Hide if necessary
        self.visible(self.config["show_toolbar"])

    def start(self):
        for widget in self.change_sensitivity:
            self.window.main_glade.get_widget(widget).set_sensitive(True)
        self.update_buttons()

    def stop(self):
        for widget in self.change_sensitivity:
            self.window.main_glade.get_widget(widget).set_sensitive(False)

    def visible(self, visible):
        if visible:
            self.toolbar.show()
        else:
            self.toolbar.hide()

        self.config["show_toolbar"] = visible

    def add_toolbutton(self, callback, label=None, image=None, stock=None,
                                                         tooltip=None):
        """Adds a toolbutton to the toolbar"""
        # Create the button
        toolbutton = gtk.ToolButton()
        if stock is not None:
            toolbutton.set_stock_id(stock)
        if label is not None:
            toolbutton.set_label(label)
        if image is not None:
            toolbutton.set_icon_widget(image)
        # Set the tooltip
        if tooltip is not None:
            toolbutton.set_tooltip_text(tooltip)
        # Connect the 'clicked' event callback
        toolbutton.connect("clicked", callback)

        # Append the button to the toolbar
        self.toolbar.insert(toolbutton, -1)

        # Show the new toolbutton
        toolbutton.show_all()

        return toolbutton

    def add_separator(self, position=None):
        """Adds a separator toolitem"""
        sep = gtk.SeparatorToolItem()
        if position is not None:
            self.toolbar.insert(sep, position)
        else:
            # Append the separator
            self.toolbar.insert(sep, -1)

        sep.show()

        return sep

    def remove(self, widget):
        """Removes a widget from the toolbar"""
        self.toolbar.remove(widget)

    ### Callbacks ###
    def on_toolbutton_add_clicked(self, data):
        log.debug("on_toolbutton_add_clicked")
        # Use the menubar's callback
        component.get("MenuBar").on_menuitem_addtorrent_activate(data)

    def on_toolbutton_remove_clicked(self, data):
        log.debug("on_toolbutton_remove_clicked")
        # Use the menubar's callbacks
        component.get("MenuBar").on_menuitem_remove_activate(data)

    def on_toolbutton_pause_clicked(self, data):
        log.debug("on_toolbutton_pause_clicked")
        # Use the menubar's callbacks
        component.get("MenuBar").on_menuitem_pause_activate(data)

    def on_toolbutton_resume_clicked(self, data):
        log.debug("on_toolbutton_resume_clicked")
        # Use the menubar's calbacks
        component.get("MenuBar").on_menuitem_resume_activate(data)

    def on_toolbutton_preferences_clicked(self, data):
        log.debug("on_toolbutton_preferences_clicked")
        # Use the menubar's callbacks
        component.get("MenuBar").on_menuitem_preferences_activate(data)

    def on_toolbutton_connectionmanager_clicked(self, data):
        log.debug("on_toolbutton_connectionmanager_clicked")
        # Use the menubar's callbacks
        component.get("MenuBar").on_menuitem_connectionmanager_activate(data)

    def on_toolbutton_queue_up_clicked(self, data):
        log.debug("on_toolbutton_queue_up_clicked")
        component.get("MenuBar").on_menuitem_queue_up_activate(data)

    def on_toolbutton_queue_down_clicked(self, data):
        log.debug("on_toolbutton_queue_down_clicked")
        component.get("MenuBar").on_menuitem_queue_down_activate(data)

    def update_buttons(self, action=None, torrent_id=None):
        if action == None:
            # If all the selected torrents are paused, then disable the 'Pause'
            # button.
            # The same goes for the 'Resume' button.
            pause = False
            resume = False

            selected = component.get('TorrentView').get_selected_torrents()
            if not selected:
                selected = []

            for torrent in selected:
                try:
                    status = component.get("TorrentView").get_torrent_status(torrent)['state']
                except KeyError, e:
                    log.debug("Error getting torrent state: %s", e)
                    continue
                if status == "Paused" or status == "Error":
                    resume = True
                else:
                    pause = True
                if pause and resume:
                    break

            # Enable the 'Remove Torrent' button only if there's some selected
            # torrent.
            remove = (len(selected) > 0)

            for name, sensitive in (("toolbutton_pause", pause),
                                    ("toolbutton_resume", resume),
                                    ("toolbutton_remove", remove)):
                self.window.main_glade.get_widget(name).set_sensitive(sensitive)


            return False

        pause = False
        resume = False
        if action == "paused":
            pause = False
            resume = True
        elif action == "resumed":
            pause = True
            resume = False
        selected = component.get('TorrentView').get_selected_torrents()
        if torrent_id == None or (torrent_id in selected and len(selected) == 1):
            self.window.main_glade.get_widget("toolbutton_pause").set_sensitive(pause)
            self.window.main_glade.get_widget("toolbutton_resume").set_sensitive(resume)
        else:
            self.update_buttons()

        return False



# -*- coding: utf-8 -*-
# aboutdialog.py
#
# Copyright (C) 2007 Marcos Pinto ('markybob') <markybob@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


import pygtk
pygtk.require('2.0')
import gtk
import pkg_resources

import deluge.common
import deluge.ui.gtkui.common as common
from deluge.ui.client import aclient as client

class AboutDialog:
    def __init__(self):
        # Get the glade file for the about dialog
        def url_hook(dialog, url):
            deluge.common.open_url_in_browser(url)
        gtk.about_dialog_set_url_hook(url_hook)
        self.about = gtk.AboutDialog()
        self.about.set_position(gtk.WIN_POS_CENTER)
        self.about.set_name("Deluge")
        self.about.set_program_name("Deluge")

        # Get the version and revision numbers
        rev = deluge.common.get_revision()
        version = deluge.common.get_version()
        if rev != "":
            version = version + "r" + rev
        self.about.set_copyright(u'Copyright \u00A9 2007-2009 Andrew Resch')
        self.about.set_comments("A peer-to-peer file sharing program\nutilizing the Bittorrent protocol.")
        self.about.set_version(version)
        self.about.set_authors(["Andrew Resch", "",
            "NullUI (replaced by ConsoleUI):", "Sadrul Habib Chowdhury", "", "WebUI:", "Martijn Voncken", "", "ConsoleUI:", "Ido Abramovich", "", "AjaxUI:", "Damien Churchill", "", "libtorrent (www.libtorrent.org):", "Arvid Norberg", "", "Past Developers:", "Zach Tibbitts", "Alon Zakai", "Marcos Pinto", "Alex Dedul"])
        self.about.set_artists(["Andrew Wedderburn", "Andrew Resch"])
        self.about.set_translator_credits("Aaron Wang Shi \nabbigss \nABCdatos \nAbcx \nActam \nAdam \nadaminikisi \nadi_oporanu \nAdrian Goll \nafby \nAhmades \nAhmad Farghal \nAhmad Gharbeia أحمد غربية \nakira \nAki Sivula \nAlan Pepelko \nAlberto \nAlberto Ferrer \nalcatr4z \nAlckO \nAleksej Korgenkov \nAlessio Treglia \nAlexander Ilyashov \nAlexander Matveev \nAlexander Saltykov \nAlexander Taubenkorb \nAlexander Telenga \nAlexander Yurtsev \nAlexandre Martani \nAlexandre Rosenfeld \nAlexandre Sapata Carbonell \nAlexey Osipov \nAlin Claudiu Radut \nallah \nAlSim \nAlvaro Carrillanca P. \nA.Matveev \nAndras Hipsag \nAndrás Kárász \nAndrea Ratto \nAndreas Johansson \nAndreas Str \nAndré F. Oliveira \nAndreiF \nandrewh \nAngel Guzman Maeso \nAníbal Deboni Neto \nanimarval \nAntonio Cono \nantoniojreyes \nAnton Shestakov \nAnton Yakutovich \nantou \nArkadiusz Kalinowski \nArtin \nartir \nAstur \nAthanasios Lefteris \nAthmane MOKRAOUI (ButterflyOfFire) \nAugusta Carla Klug \nAvoledo Marco \naxaard \nAxelRafn \nAxezium \nAyont \nb3rx \nBae Taegil \nBajusz Tamás \nBalaam's Miracle \nBallestein \nBent Ole Fosse \nberto89 \nbigx \nBjorn Inge Berg \nblackbird \nBlackeyed \nblackmx \nBlueSky \nBlutheo \nbmhm \nbob00work \nboenki \nBogdan Bădic-Spătariu \nbonpu \nBoone \nboss01 \nBranislav Jovanović \nbronze \nbrownie \nBrus46 \nbumper \nbutely \nBXCracer \nc0nfidencal \nCan Kaya \nCarlos Alexandro Becker \ncassianoleal \nCédric.h \nCésar Rubén \nchaoswizard \nChen Tao \nchicha \nChien Cheng Wei \nChristian Kopac \nChristian Widell \nChristoffer Brodd-Reijer \nchristooss \nCityAceE \nClopy \nClusty \ncnu \nCommandant \nConstantinos Koniaris \nCoolmax \ncosmix \nCostin Chirvasuta \nCoVaLiDiTy \ncow_2001 \nCrispin Kirchner \ncrom \nCruster \nCybolic \nDan Bishop \nDanek \nDani \nDaniel Demarco \nDaniel Ferreira \nDaniel Frank \nDaniel Holm \nDaniel Høyer Iversen \nDaniel Marynicz \nDaniel Nylander \nDaniel Patriche \nDaniel Schildt \nDaniil Sorokin \nDante Díaz \nDaria Michalska \nDarkenCZ \nDarren \nDaspah \nDavid Eurenius \ndavidhjelm \nDavid Machakhelidze \nDawid Dziurdzia \nDaya Adianto \ndcruz \nDeady \nDereck Wonnacott \nDevgru \nDevid Antonio Filoni \nDevilDogTG \ndi0rz` \nDialecti Valsamou \nDiego Medeiros \nDkzoffy \nDmitrij D. Czarkoff \nDmitriy Geels \nDmitry Olyenyov \nDominik Kozaczko \nDominik Lübben \ndoomster \nDorota Król \nDoyen Philippe \nDread Knight \nDreamSonic \nduan \nDuong Thanh An \nDvoglavaZver \ndwori \ndylansmrjones \nEbuntor \nEdgar Alejandro Jarquin Flores \nEetu \nekerazha \nElias Julkunen \nelparia \nEmberke \nEmiliano Goday Caneda \nEndelWar \neng.essam \nenubuntu \nercangun \nErdal Ronahi \nergin üresin \nEric \nÉric Lassauge \nErlend Finvåg \nErrdil \nethan shalev \nEvgeni Spasov \nezekielnin \nFabian Ordelmans \nFabio Mazanatti \nFábio Nogueira \nFaCuZ \nFelipe Lerena \nFernando Pereira \nfjetland \nFlorian Schäfer \nFoBoS \nFolke \nForce \nfosk \nfragarray \nfreddeg \nFrédéric Perrin \nFredrik Kilegran \nFreeAtMind \nFulvio Ciucci \nGabor Kelemen \nGalatsanos Panagiotis \nGaussian \ngdevitis \nGeorg Brzyk \nGeorge Dumitrescu \nGeorgi Arabadjiev \nGeorg Sieber \nGerd Radecke \nGermán Heusdens \nGianni Vialetto \nGigih Aji Ibrahim \nGiorgio Wicklein \nGiovanni Rapagnani \nGiuseppe \ngl \nglen \ngranjerox \nGreen Fish \ngreentea \nGreyhound \nG. U. \nGuillaume BENOIT \nGuillaume Pelletier \nGustavo Henrique Klug \ngutocarvalho \nGuybrush88 \nHans Rødtang \nHardDisk \nHargas Gábor \nHeitor Thury Barreiros Barbosa \nhelios91940 \nhelix84 \nHelton Rodrigues \nHendrik Luup \nHenrique Ferreiro \nHenry Goury-Laffont \nHezy Amiel \nhidro \nhoball \nhokten \nHolmsss \nhristo.num \nHubert Życiński \nHyo \nIarwain \nibe \nibear \nId2ndR \nIgor Zubarev \nIKON (Ion) \nimen \nIonuț Jula \nIsabelle STEVANT \nIstván Nyitrai \nIvan Petrovic \nIvan Prignano \nIvaSerge \njackmc \nJacks0nxD \nJack Shen \nJacky Yeung \nJacques Stadler \nJanek Thomaschewski \nJan Kaláb \nJan Niklas Hasse \nJasper Groenewegen \nJavi Rodríguez \nJayasimha (ಜಯಸಿಂಹ) \njeannich \nJeff Bailes \nJesse Zilstorff \nJoan Duran \nJoão Santos \nJoar Bagge \nJoe Anderson \nJoel Calado \nJohan Linde \nJohn Garland \nJojan \njollyr0ger \nJonas Bo Grimsgaard \nJonas Granqvist \nJonas Slivka \nJonathan Zeppettini \nJørgen \nJørgen Tellnes \njosé \nJosé Geraldo Gouvêa \nJosé Iván León Islas \nJosé Lou C. \nJose Sun \nJr. \nJukka Kauppinen \nJulián Alarcón \njulietgolf \nJusic \nJustzupi \nKaarel \nKai Thomsen \nKalman Tarnay \nKamil Páral \nKane_F \nkaotiks@gmail.com \nKateikyoushii \nkaxhinaz \nKazuhiro NISHIYAMA \nKerberos \nKeresztes Ákos \nkevintyk \nkiersie \nKimbo^ \nKim Lübbe \nkitzOgen \nKjetil Rydland \nkluon \nkmikz \nKnedlyk \nkoleoptero \nKőrösi Krisztián \nKouta \nKrakatos \nKrešo Kunjas \nkripken \nKristaps \nKristian Øllegaard \nKristoffer Egil Bonarjee \nKrzysztof Janowski \nKrzysztof Zawada \nLarry Wei Liu \nlaughterwym \nLaur Mõtus \nlazka \nleandrud \nlê bình \nLe Coz Florent \nLeo \nliorda \nLKRaider \nLoLo_SaG \nLong Tran \nLorenz \nLow Kian Seong \nLuca Andrea Rossi \nLuca Ferretti \nLucky LIX \nLuis Gomes \nLuis Reis \nŁukasz Wyszyński \nluojie-dune \nmaaark \nMaciej Chojnacki \nMaciej Meller \nMads Peter Rommedahl \nMajor Kong \nMalaki \nmalde \nMalte Lenz \nMantas Kriaučiūnas \nMara Sorella \nMarcin \nMarcin Falkiewicz \nmarcobra \nMarco da Silva \nMarco de Moulin \nMarco Rodrigues \nMarcos \nMarcos Escalier \nMarcos Pinto \nMarcus Ekstrom \nMarek Dębowski \nMário Buči \nMario Munda \nMarius Andersen \nMarius Hudea \nMarius Mihai \nMariusz Cielecki \nMark Krapivner \nmarko-markovic \nMarkus Brummer \nMarkus Sutter \nMartin \nMartin Dybdal \nMartin Iglesias \nMartin Lettner \nMartin Pihl \nMasoud Kalali \nmat02 \nMatej Urbančič \nMathias-K \nMathieu Arès \nMathieu D. (MatToufoutu) \nMathijs \nMatrik \nMatteo Renzulli \nMatteo Settenvini \nMatthew Gadd \nMatthias Benkard \nMatthias Mailänder \nMattias Ohlsson \nMauro de Carvalho \nMax Molchanov \nMe \nMercuryCC \nMert Bozkurt \nMert Dirik \nMFX \nmhietar \nmibtha \nMichael Budde \nMichael Kaliszka \nMichalis Makaronides \nMichał Tokarczyk \nMiguel Pires da Rosa \nMihai Capotă \nMiika Metsälä \nMikael Fernblad \nMike Sierra \nmikhalek \nMilan Prvulović \nMilo Casagrande \nMindaugas \nMiroslav Matejaš \nmisel \nmithras \nMitja Pagon \nM.Kitchen \nMohamed Magdy \nmoonkey \nMrBlonde \nmuczy \nMünir Ekinci \nMustafa Temizel \nmvoncken \nMytonn \nNagyMarton \nneaion \nNeil Lin \nNemo \nNerijus Arlauskas \nNicklas Larsson \nNicolaj Wyke \nNicola Piovesan \nNicolas Sabatier \nNicolas Velin \nNightfall \nNiKoB \nNikolai M. Riabov \nNiko_Thien \nniska \nNithir \nnoisemonkey \nnomemohes \nnosense \nnull \nNuno Estêvão \nNuno Santos \nnxxs \nnyo \nobo \nOjan \nOlav Andreas Lindekleiv \noldbeggar \nOlivier FAURAX \norphe \nosantana \nOsman Tosun \nOssiR \notypoks \nounn \nOz123 \nÖzgür BASKIN \nPablo Carmona A. \nPablo Ledesma \nPablo Navarro Castillo \nPaco Molinero \nPål-Eivind Johnsen \npano \nPaolo Naldini \nParacelsus \nPatryk13_03 \nPatryk Skorupa \nPattogoTehen \nPaul Lange \nPavcio \nPaweł Wysocki \nPedro Brites Moita \nPedro Clemente Pereira Neto \nPekka \"PEXI\" Niemistö \nPenegal \nPenzo \nperdido \nPeter Kotrcka \nPeter Skov \nPeter Van den Bosch \nPetter Eklund \nPetter Viklund \nphatsphere \nPhenomen \nPhilipi \nPhilippides Homer \nphoenix \npidi \nPierre Quillery \nPierre Rudloff \nPierre Slamich \nPietrao \nPiotr Strębski \nPiotr Wicijowski \nPittmann Tamás \nPlaymolas \nPrescott \nPrescott_SK \npronull \nPrzemysław Kulczycki \nPumy \npushpika \nPY \nqubicllj \nr21vo \nRafał Barański \nrainofchaos \nRajbir \nras0ir \nRat \nrd1381 \nRenato \nRene Hennig \nRene Pärts \nRicardo Duarte \nRichard \nRobert Hrovat \nRoberth Sjonøy \nRobert Lundmark \nRobin Jakobsson \nRobin Kåveland \nRodrigo Donado \nRoel Groeneveld \nrohmaru \nRolf Christensen \nRolf Leggewie \nRoni Kantis \nRonmi \nRostislav Raykov \nroyto \nRuiAmaro \nRui Araújo \nRui Moura \nRune Svendsen \nRusna \nRytis \nSabirov Mikhail \nsalseeg \nSami Koskinen \nSamir van de Sand \nSamuel Arroyo Acuña \nSamuel R. C. Vale \nSanel \nSanti \nSanti Martínez Cantelli \nSardan \nSargate Kanogan \nSarmad Jari \nSaša Bodiroža \nsat0shi \nSaulius Pranckevičius \nSavvas Radevic \nSebastian Krauß \nSebastián Porta \nSedir \nSefa Denizoğlu \nsekolands \nSelim Suerkan \nsemsomi \nSergii Golovatiuk \nsetarcos \nSheki \nShironeko \nShlomil \nsilfiriel \nSimone Tolotti \nSimone Vendemia \nsirkubador \nSławomir Więch \nslip \nslyon \nsmoke \nSonja \nspectral \nspin_555 \nspitf1r3 \nSpiziuz \nSpyros Theodoritsis \nSqUe \nSquigly \nsrtck \nStefan Horning \nStefano Maggiolo \nStefano Roberto Soleti \nsteinberger \nStéphane Travostino \nStephan Klein \nSteven De Winter \nStevie \nStian24 \nstylius \nSukarn Maini \nSunjae Park \nSusana Pereira \nszymon siglowy \ntakercena \nTAS \nTaygeto \ntemy4 \ntexxxxxx \nthamood \nThanos Chatziathanassiou \nTharawut Paripaiboon \nTheodoor \nThéophane Anestis \nThor Marius K. Høgås \nTiago Silva \nTiago Sousa \nTikkel \ntim__b \nTim Bordemann \nTim Fuchs \nTim Kornhammar \nTimo \nTimo Jyrinki \nTimothy Babych \nTitkosRejtozo \nTom \nTomas Gustavsson \nTomas Valentukevičius \nTomasz Dominikowski \nTomislav Plavčić \nTom Mannerhagen \nTommy Mikkelsen \nTom Verdaat \nTony Manco \nTor Erling H. Opsahl \nToudi \ntqm_z \nTrapanator \nTribaal \nTriton \nTuniX12 \nTuomo Sipola \nturbojugend_gr \nTurtle.net \ntwilight \ntymmej \nUlrik \nUmarzuki Mochlis \nunikob \nVadim Gusev \nVagi \nValentin Bora \nValmantas Palikša \nVASKITTU \nVassilis Skoullis \nvetal17 \nvicedo \nviki \nvillads hamann \nVincent Garibal \nVincent Ortalda \nvinchi007 \nVinícius de Figueiredo Silva \nVinzenz Vietzke \nvirtoo \nvirtual_spirit \nVitor Caike \nVitor Lamas Gatti \nVladimir Lazic \nVladimir Sharshov \nWanderlust \nWander Nauta \nWard De Ridder \nWebCrusader \nwebdr \nWentao Tang \nwilana \nWilfredo Ernesto Guerrero Campos \nWim Champagne \nWorld Sucks \nXabi Ezpeleta \nXavi de Moner \nXavierToo \nXChesser \nXiaodong Xu \nxyb \nYaron \nYasen Pramatarov \nYesPoX \nYuren Ju \nYves MATHIEU \nzekopeko \nzhuqin \nZissan \nΓιάννης Κατσαμπίρης \nАртём Попов \nМиша \nШаймарданов Максим \n蔡查理")
        self.about.set_wrap_license(True)
        self.about.set_license(_("This program is free software; you can redistribute \
it and/or modify it under the terms of the GNU General Public License as published by \
the Free Software Foundation; either version 3 of the License, or (at your option) any \
later version. This program is distributed in the hope that it will be useful, but \
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS \
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You \
should have received a copy of the GNU General Public License along with this program; \
if not, see <http://www.gnu.org/licenses>."))
        self.about.set_website("http://deluge-torrent.org")
        self.about.set_website_label("http://deluge-torrent.org")
        self.about.set_icon(common.get_logo(32))
        self.about.set_logo(gtk.gdk.pixbuf_new_from_file(
                                deluge.common.get_pixmap("deluge-about.png")))

    def run(self):
        self.about.show_all()
        self.about.run()
        self.about.destroy()



#!/usr/bin/env python
#
# help.py
#
# Copyright (C) 2008-2009 Ido Abramovich <ido.deluge@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#
from deluge.ui.console import UI_PATH
from deluge.ui.console.main import BaseCommand, load_commands
from deluge.ui.console.colors import templates
import os

class Command(BaseCommand):
    """displays help on other commands"""

    usage =  "Usage: help [command]"

    def __init__(self):
        BaseCommand.__init__(self)
        # get a list of commands, exclude 'help' so we won't run into a recursive loop.
        self._commands = load_commands(os.path.join(UI_PATH,'commands'), exclude=['help'])
        self._commands['help'] = self

    def handle(self, *args, **options):
        if args:
            if len(args) > 1:
                print usage
                return
            try:
                cmd = self._commands[args[0]]
            except KeyError:
                print templates.ERROR('unknown command %r' % args[0])
                return
            try:
                parser = cmd.create_parser()
                print parser.format_help()
            except AttributeError, e:
                print cmd.__doc__ or 'No help for this command'
        else:
            max_length = max( len(k) for k in self._commands)
            for cmd in sorted(self._commands):
                print templates.help(max_length, cmd, self._commands[cmd].__doc__ or '')
            print
            print 'for help on a specific command, use "<command> --help"'

    def complete(self, text, *args):
        return [ x for x in self._commands.keys() if x.startswith(text) ]



#
# webuipluginbase.py
#
# Copyright (C) 2008 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#



from deluge.log import LOG as log
from deluge import component
import os
api = component.get("WebPluginApi")


class WebUIPluginBase:
    """
    convenience class, you don't have to use this, but it does make things easyer to setup.
    * static files: /data is automatically served on http as as /safe-plugin-name/data
    * templates: /template are added to api.render.plugin-name.
    * pages: urls attribute registers pages : urls = [(url, class), ..]
    """
    include_javascript = []
    ajax_javascript = []
    urls = []

    def __init__(self, plugin_api, plugin_name):
        log.debug("%s plugin : start initalize.." % plugin_name)

        self.plugin = plugin_api
        self.plugin_name = plugin_name
        self.clean_plugin_name = plugin_name.lower().replace(" ","_")


    def base_enable(self):
        """
        enable plugin.
        """
        for url , klass in self.urls:
            api.page_manager.register_page(url, klass)

        for js in self.include_javascript:
            api.page_manager.include_javascript.append(js)

        class egg_data_static(api.egg_handler): #serves files in /data from egg
            resource = self.clean_plugin_name
            base_path = "data"

        #use as : api.render.plugin-name.template-name[excluding.html](parameters)
        setattr(api.render, self.clean_plugin_name, api.egg_render(self.clean_plugin_name, "template"))

        api.page_manager.register_page("/%s/data/(.*)" % self.clean_plugin_name , egg_data_static, use_module=False)

        log.debug("%s plugin : end base_enable().." % self.plugin_name)

    def base_disable(self):
        """
        disable plugin.
        """
        for url , klass in self.urls:
            api.page_manager.deregister_page(url, klass)

        for js in self.include_javascript:
            api.page_manager.include_javascript.remove(js)

        log.debug("%s plugin : end base_disable().." % self.plugin_name)






#
# connectionmanager.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


import gtk, gtk.glade
import pkg_resources
import gobject
import socket
import os
import subprocess
import time
import threading
import urlparse

import deluge.component as component
import deluge.xmlrpclib as xmlrpclib
import deluge.common
import deluge.ui.gtkui.common as common
from deluge.ui.common import get_localhost_auth_uri
from deluge.ui.client import aclient as client
import deluge.configmanager
from deluge.configmanager import ConfigManager
from deluge.log import LOG as log

DEFAULT_URI = "http://127.0.0.1:58846"
DEFAULT_HOST = DEFAULT_URI.split(":")[1][2:]
DEFAULT_PORT = DEFAULT_URI.split(":")[-1]

DEFAULT_CONFIG = {
    "hosts": [DEFAULT_URI]
}

HOSTLIST_COL_PIXBUF = 0
HOSTLIST_COL_URI = 1
HOSTLIST_COL_STATUS = 2

HOSTLIST_STATUS = [
    "Offline",
    "Online",
    "Connected"
]

HOSTLIST_PIXBUFS = [
    # This is populated in ConnectionManager.__init__
]

if deluge.common.windows_check():
    import win32api


def cell_render_host(column, cell, model, row, data):
    host = model[row][data]
    u = urlparse.urlsplit(host)
    if not u.hostname:
        host = "http://" + host
    u = urlparse.urlsplit(host)
    if u.username:
        text = u.username + "@" + u.hostname + ":" + str(u.port)
    else:
        text = u.hostname + ":" + str(u.port)

    cell.set_property('text', text)

class ConnectionManager(component.Component):
    def __init__(self):
        component.Component.__init__(self, "ConnectionManager")
        # Get the glade file for the connection manager
        self.glade = gtk.glade.XML(
                    pkg_resources.resource_filename("deluge.ui.gtkui",
                                            "glade/connection_manager.glade"))

        self.window = component.get("MainWindow")
        self.config = ConfigManager("hostlist.conf.1.1", DEFAULT_CONFIG)

        self.gtkui_config = ConfigManager("gtkui.conf")
        self.connection_manager = self.glade.get_widget("connection_manager")
        # Make the Connection Manager window a transient for the main window.
        self.connection_manager.set_transient_for(self.window.window)

        # Create status pixbufs
        for stock_id in (gtk.STOCK_NO, gtk.STOCK_YES, gtk.STOCK_CONNECT):
            HOSTLIST_PIXBUFS.append(self.connection_manager.render_icon(stock_id, gtk.ICON_SIZE_MENU))

        self.hostlist = self.glade.get_widget("hostlist")
        self.connection_manager.set_icon(common.get_logo(32))

        self.glade.get_widget("image1").set_from_pixbuf(common.get_logo(32))

        # connection status pixbuf, hostname:port, status
        self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, int)

        # Holds the online status of hosts
        self.online_status = {}

        # Fill in hosts from config file
        for host in self.config["hosts"]:
            row = self.liststore.append()
            self.liststore.set_value(row, HOSTLIST_COL_URI, host)

        # Setup host list treeview
        self.hostlist.set_model(self.liststore)
        render = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn(
            "Status", render, pixbuf=HOSTLIST_COL_PIXBUF)
        self.hostlist.append_column(column)
        render = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Host", render, text=HOSTLIST_COL_URI)
        column.set_cell_data_func(render, cell_render_host, 1)
        self.hostlist.append_column(column)

        self.glade.signal_autoconnect({
            "on_button_addhost_clicked": self.on_button_addhost_clicked,
            "on_button_removehost_clicked": self.on_button_removehost_clicked,
            "on_button_startdaemon_clicked": \
                self.on_button_startdaemon_clicked,
            "on_button_close_clicked": self.on_button_close_clicked,
            "on_button_connect_clicked": self.on_button_connect_clicked,
            "on_chk_autoconnect_toggled": self.on_chk_autoconnect_toggled,
            "on_chk_autostart_toggled": self.on_chk_autostart_toggled,
            "on_chk_donotshow_toggled": self.on_chk_donotshow_toggled
        })

        self.connection_manager.connect("delete-event", self.on_delete_event)
        # Connect to the 'changed' event of TreeViewSelection to get selection
        # changes.
        self.hostlist.get_selection().connect("changed",
                                    self.on_selection_changed)

        self.hostlist.connect("row-activated", self._on_row_activated)

        # If classic mode is set, we just start up a localhost daemon and connect to it
        if self.gtkui_config["classic_mode"]:
            self.start_localhost(DEFAULT_PORT)
            # We need to wait for the host to start before connecting
            uri = None
            while not uri:
                # We need to keep trying because the daemon may not have been started yet
                # and the 'auth' file may not have been created
                uri = get_localhost_auth_uri(DEFAULT_URI)
                time.sleep(0.01)

            while not self.test_online_status(uri):
                time.sleep(0.01)
            client.set_core_uri(uri)
            self.hide()
            return

        # This controls the timer, if it's set to false the update timer will stop.
        self._do_update = True
        self._update_list()

        # Auto connect to a host if applicable
        if self.gtkui_config["autoconnect"] and \
            self.gtkui_config["autoconnect_host_uri"] != None:
            uri = self.gtkui_config["autoconnect_host_uri"]
            if self.test_online_status(uri):
                # Host is online, so lets connect
                client.set_core_uri(uri)
                self.hide()
            elif self.gtkui_config["autostart_localhost"]:
                # Check to see if we are trying to connect to a localhost
                u = urlparse.urlsplit(uri)
                if u.hostname == "localhost" or u.hostname == "127.0.0.1":
                    # This is a localhost, so lets try to start it
                    # First add it to the list
                    self.add_host("localhost", u.port)
                    self.start_localhost(u.port)
                    # Get the localhost uri with authentication details
                    auth_uri = None
                    while not auth_uri:
                        # We need to keep trying because the daemon may not have been started yet
                        # and the 'auth' file may not have been created
                        auth_uri = get_localhost_auth_uri(uri)
                        time.sleep(0.01)

                    # We need to wait for the host to start before connecting
                    while not self.test_online_status(auth_uri):
                        time.sleep(0.01)
                    client.set_core_uri(auth_uri)
                    self.hide()

    def start(self):
        if self.gtkui_config["autoconnect"]:
            # We need to update the autoconnect_host_uri on connection to host
            # start() gets called whenever we get a new connection to a host
            self.gtkui_config["autoconnect_host_uri"] = client.get_core_uri()

    def show(self):
        # Set the checkbuttons according to config
        self.glade.get_widget("chk_autoconnect").set_active(
            self.gtkui_config["autoconnect"])
        self.glade.get_widget("chk_autostart").set_active(
            self.gtkui_config["autostart_localhost"])
        self.glade.get_widget("chk_donotshow").set_active(
            not self.gtkui_config["show_connection_manager_on_start"])

        # Setup timer to update host status
        self._update_timer = gobject.timeout_add(1000, self._update_list)
        self._update_list()
        self._update_list()
        self.connection_manager.show_all()

    def hide(self):
        self.connection_manager.hide()
        self._do_update = False
        try:
            gobject.source_remove(self._update_timer)
        except AttributeError:
            # We are probably trying to hide the window without having it showed
            # first.  OK to ignore.
            pass

    def _update_list(self):
        """Updates the host status"""
        def update_row(model=None, path=None, row=None, columns=None):
            uri = model.get_value(row, HOSTLIST_COL_URI)
            threading.Thread(target=self.test_online_status, args=(uri,)).start()
            try:
                online = self.online_status[uri]
            except:
                online = False

            # Update hosts status
            if online:
                online = HOSTLIST_STATUS.index("Online")
            else:
                online = HOSTLIST_STATUS.index("Offline")

            if urlparse.urlsplit(uri).hostname == "localhost" or urlparse.urlsplit(uri).hostname == "127.0.0.1":
                uri = get_localhost_auth_uri(uri)

            if uri == current_uri:
                online = HOSTLIST_STATUS.index("Connected")

            model.set_value(row, HOSTLIST_COL_STATUS, online)
            model.set_value(row, HOSTLIST_COL_PIXBUF, HOSTLIST_PIXBUFS[online])

        current_uri = client.get_core_uri()
        self.liststore.foreach(update_row)
        # Update the buttons
        self.update_buttons()

        # See if there is any row selected
        paths = self.hostlist.get_selection().get_selected_rows()[1]
        if len(paths) < 1:
            # And there is at least 1 row
            if self.liststore.iter_n_children(None) > 0:
                # Then select the first row
                self.hostlist.get_selection().select_iter(self.liststore.get_iter_first())
        return self._do_update

    def update_buttons(self):
        """Updates the buttons based on selection"""
        if self.liststore.iter_n_children(None) < 1:
            # There is nothing in the list
            self.glade.get_widget("button_startdaemon").set_sensitive(True)
            self.glade.get_widget("button_connect").set_sensitive(False)
            self.glade.get_widget("button_removehost").set_sensitive(False)
            self.glade.get_widget("image_startdaemon").set_from_stock(
                gtk.STOCK_EXECUTE, gtk.ICON_SIZE_MENU)
            self.glade.get_widget("label_startdaemon").set_text(
                "_Start Daemon")
        self.glade.get_widget("label_startdaemon").set_use_underline(
            True)

        # Get the selected row's URI
        paths = self.hostlist.get_selection().get_selected_rows()[1]
        # If nothing is selected, just return
        if len(paths) < 1:
            return
        row = self.liststore.get_iter(paths[0])
        uri = self.liststore.get_value(row, HOSTLIST_COL_URI)
        status = self.liststore.get_value(row, HOSTLIST_COL_STATUS)

        # Check to see if a localhost is selected
        localhost = False
        u = urlparse.urlsplit(uri)
        if u.hostname == "localhost" or u.hostname == "127.0.0.1":
            localhost = True

        # Make sure buttons are sensitive at start
        self.glade.get_widget("button_startdaemon").set_sensitive(True)
        self.glade.get_widget("button_connect").set_sensitive(True)
        self.glade.get_widget("button_removehost").set_sensitive(True)

        # See if this is the currently connected URI
        if status == HOSTLIST_STATUS.index("Connected"):
            # Display a disconnect button if we're connected to this host
            self.glade.get_widget("button_connect").set_label("gtk-disconnect")
            self.glade.get_widget("button_removehost").set_sensitive(False)
        else:
            self.glade.get_widget("button_connect").set_label("gtk-connect")
            if status == HOSTLIST_STATUS.index("Offline") and not localhost:
                self.glade.get_widget("button_connect").set_sensitive(False)

        # Check to see if the host is online
        if status == HOSTLIST_STATUS.index("Connected") \
            or status == HOSTLIST_STATUS.index("Online"):
            self.glade.get_widget("image_startdaemon").set_from_stock(
                gtk.STOCK_STOP, gtk.ICON_SIZE_MENU)
            self.glade.get_widget("label_startdaemon").set_text(
                "_Stop Daemon")

        # Update the start daemon button if the selected host is localhost
        if localhost and status == HOSTLIST_STATUS.index("Offline"):
            # The localhost is not online
            self.glade.get_widget("image_startdaemon").set_from_stock(
                gtk.STOCK_EXECUTE, gtk.ICON_SIZE_MENU)
            self.glade.get_widget("label_startdaemon").set_text(
                "_Start Daemon")

        if not localhost:
            # An offline host
            self.glade.get_widget("button_startdaemon").set_sensitive(False)

        # Make sure label is displayed correctly using mnemonics
        self.glade.get_widget("label_startdaemon").set_use_underline(
            True)

    def save(self):
        """Save the current host list to file"""
        def append_row(model=None, path=None, row=None, columns=None):
            hostlist.append(model.get_value(row, HOSTLIST_COL_URI))

        hostlist = []
        self.liststore.foreach(append_row, hostlist)
        self.config["hosts"] = hostlist
        self.config.save()

    def test_online_status(self, uri):
        """Tests the status of URI.. Returns True or False depending on status.
        """
        online = True
        host = None
        try:
            u = urlparse.urlsplit(uri)
            if u.hostname == "localhost" or u.hostname == "127.0.0.1":
                host = xmlrpclib.ServerProxy(get_localhost_auth_uri(uri))
            else:
                host = xmlrpclib.ServerProxy(uri)
            host.ping()
        except Exception:
            online = False

        del host
        self.online_status[uri] = online
        return online

    ## Callbacks
    def on_delete_event(self, widget, event):
        self.hide()
        return True

    def on_button_addhost_clicked(self, widget):
        log.debug("on_button_addhost_clicked")
        dialog = self.glade.get_widget("addhost_dialog")
        dialog.set_icon(common.get_logo(16))
        hostname_entry = self.glade.get_widget("entry_hostname")
        port_spinbutton = self.glade.get_widget("spinbutton_port")
        username_entry = self.glade.get_widget("entry_username")
        password_entry = self.glade.get_widget("entry_password")
        response = dialog.run()
        if response == 1:
            username = username_entry.get_text()
            password = password_entry.get_text()
            hostname = hostname_entry.get_text()
            if not urlparse.urlsplit(hostname).hostname:
                # We need to add a http://
                hostname = "http://" + hostname
            u = urlparse.urlsplit(hostname)
            if username and password:
                host = u.scheme + "://" + username + ":" + password + "@" + u.hostname
            else:
                host = hostname

            # We add the host
            self.add_host(host, port_spinbutton.get_value_as_int())

        dialog.hide()

    def add_host(self, hostname, port):
        """Adds the host to the list"""
        if not urlparse.urlsplit(hostname).scheme:
            # We need to add http:// to this
            hostname = "http://" + hostname

        # Check to make sure the hostname is at least 1 character long
        if len(hostname) < 1:
            return

        # Get the port and concatenate the hostname string
        hostname = hostname + ":" + str(port)

        # Check to see if there is already an entry for this host and return
        # if thats the case
        self.hosts_liststore = []
        def each_row(model, path, iter, data):
            self.hosts_liststore.append(
                model.get_value(iter, HOSTLIST_COL_URI))
        self.liststore.foreach(each_row, None)
        if hostname in self.hosts_liststore:
            return

        # Host isn't in the list, so lets add it
        row = self.liststore.append()
        self.liststore.set_value(row, HOSTLIST_COL_URI, hostname)
        # Save the host list to file
        self.save()
        # Update the status of the hosts
        self._update_list()

    def on_button_removehost_clicked(self, widget):
        log.debug("on_button_removehost_clicked")
        # Get the selected rows
        paths = self.hostlist.get_selection().get_selected_rows()[1]
        for path in paths:
            self.liststore.remove(self.liststore.get_iter(path))

        # Update the hostlist
        self._update_list()

        # Save the host list
        self.save()

    def on_button_startdaemon_clicked(self, widget):
        log.debug("on_button_startdaemon_clicked")
        if self.liststore.iter_n_children(None) < 1:
            # There is nothing in the list, so lets create a localhost entry
            self.add_host(DEFAULT_HOST, DEFAULT_PORT)
            # ..and start the daemon.
            self.start_localhost(DEFAULT_PORT)
            return

        paths = self.hostlist.get_selection().get_selected_rows()[1]
        if len(paths) < 1:
            return
        row = self.liststore.get_iter(paths[0])
        status = self.liststore.get_value(row, HOSTLIST_COL_STATUS)
        uri = self.liststore.get_value(row, HOSTLIST_COL_URI)
        u = urlparse.urlsplit(uri)
        if HOSTLIST_STATUS[status] == "Online" or\
            HOSTLIST_STATUS[status] == "Connected":
            # We need to stop this daemon
            # Call the shutdown method on the daemon
            if u.hostname == "127.0.0.1" or u.hostname == "localhost":
                uri = get_localhost_auth_uri(uri)
            core = xmlrpclib.ServerProxy(uri)
            core.shutdown()
            # Update display to show change
            self._update_list()
        elif HOSTLIST_STATUS[status] == "Offline":
            self.start_localhost(u.port)

    def start_localhost(self, port):
        """Starts a localhost daemon"""
        port = str(port)
        log.info("Starting localhost:%s daemon..", port)
        # Spawn a local daemon
        if deluge.common.windows_check():
            win32api.WinExec("deluged -p %s" % port)
        elif deluge.common.osx_check():
            subprocess.call(["nohup", "deluged", "--port=%s" % port,
                "--config=%s" % deluge.configmanager.get_config_dir()])
        else:
            subprocess.call(["deluged", "--port=%s" % port,
                "--config=%s" % deluge.configmanager.get_config_dir()])

    def on_button_close_clicked(self, widget):
        log.debug("on_button_close_clicked")
        self.hide()

    def on_button_connect_clicked(self, widget):
        log.debug("on_button_connect_clicked")
        component.stop()
        paths = self.hostlist.get_selection().get_selected_rows()[1]
        row = self.liststore.get_iter(paths[0])
        status = self.liststore.get_value(row, HOSTLIST_COL_STATUS)
        uri = self.liststore.get_value(row, HOSTLIST_COL_URI)
        # Determine if this is a localhost
        localhost = False
        u = urlparse.urlsplit(uri)
        if u.hostname == "localhost" or u.hostname == "127.0.0.1":
            localhost = True


        if status == HOSTLIST_STATUS.index("Connected"):
            # Stop all the components first.
            component.stop()
            # If we are connected to this host, then we will disconnect.
            client.set_core_uri(None)
            self._update_list()
            return

        # Test the host to see if it is online or not.  We don't use the status
        # column information because it can be up to 5 seconds out of sync.
        if not self.test_online_status(uri):
            log.warning("Host does not appear to be online..")
            # If this is an offline localhost.. lets start it and connect
            if localhost:
                self.start_localhost(u.port)
                # We need to wait for the host to start before connecting
                auth_uri = None
                while not auth_uri:
                    auth_uri = get_localhost_auth_uri(uri)
                    time.sleep(0.01)

                while not self.test_online_status(auth_uri):
                    time.sleep(0.01)
                client.set_core_uri(auth_uri)
                self._update_list()
                self.hide()

            # Update the list to show proper status
            self._update_list()

            return

        # Status is OK, so lets change to this host
        if localhost:
            client.set_core_uri(get_localhost_auth_uri(uri))
        else:
            client.set_core_uri(uri)

        self.hide()

    def on_chk_autoconnect_toggled(self, widget):
        log.debug("on_chk_autoconnect_toggled")
        value = widget.get_active()
        self.gtkui_config["autoconnect"] = value
        # If we are currently connected to a host, set that as the autoconnect
        # host.
        if client.get_core_uri() != None:
            self.gtkui_config["autoconnect_host_uri"] = client.get_core_uri()

    def on_chk_autostart_toggled(self, widget):
        log.debug("on_chk_autostart_toggled")
        value = widget.get_active()
        self.gtkui_config["autostart_localhost"] = value

    def on_chk_donotshow_toggled(self, widget):
        log.debug("on_chk_donotshow_toggled")
        value = widget.get_active()
        self.gtkui_config["show_connection_manager_on_start"] = not value

    def on_selection_changed(self, treeselection):
        log.debug("on_selection_changed")
        self.update_buttons()

    def _on_row_activated(self, tree, path, view_column):
        self.on_button_connect_clicked(self.glade.get_widget("button_connect"))



"""
Web API (wrapper around WSGI)
(from web.py)
"""

__all__ = [
    "config",
    "badrequest", "notfound", "gone", "internalerror",
    "header", "output", "flush", "debug",
    "input", "data",
    "setcookie", "cookies",
    "ctx",
    "loadhooks", "load", "unloadhooks", "unload", "_loadhooks",
    "wsgifunc"
]

import sys, os, cgi, threading, Cookie, pprint, traceback
try: import itertools
except ImportError: pass
from utils import storage, storify, threadeddict, dictadd, intget, lstrips, utf8

config = storage()
config.__doc__ = """
A configuration object for various aspects of web.py.

`db_parameters`
   : A dictionary containing the parameters to be passed to `connect`
     when `load()` is called.
`db_printing`
   : Set to `True` if you would like SQL queries and timings to be
     printed to the debug output.

"""

def badrequest():
    """Return a `400 Bad Request` error."""
    ctx.status = '400 Bad Request'
    header('Content-Type', 'text/html')
    return output('bad request')

def notfound():
    """Returns a `404 Not Found` error."""
    ctx.status = '404 Not Found'
    header('Content-Type', 'text/html')
    return output('not found')

def gone():
    """Returns a `410 Gone` error."""
    ctx.status = '410 Gone'
    header('Content-Type', 'text/html')
    return output("gone")

def internalerror():
    """Returns a `500 Internal Server` error."""
    ctx.status = "500 Internal Server Error"
    ctx.headers = [('Content-Type', 'text/html')]
    ctx.output = "internal server error"

def header(hdr, value, unique=False):
    """
    Adds the header `hdr: value` with the response.

    If `unique` is True and a header with that name already exists,
    it doesn't add a new one.
    """
    hdr, value = utf8(hdr), utf8(value)
    # protection against HTTP response splitting attack
    if '\n' in hdr or '\r' in hdr or '\n' in value or '\r' in value:
        raise ValueError, 'invalid characters in header'

    if unique is True:
        for h, v in ctx.headers:
            if h.lower() == hdr.lower(): return

    ctx.headers.append((hdr, value))

def output(string_):
    """Appends `string_` to the response."""
    if isinstance(string_, unicode): string_ = string_.encode('utf8')
    if ctx.get('flush'):
        ctx._write(string_)
    else:
        ctx.output += str(string_)

def flush():
    ctx.flush = True
    return flush

def input(*requireds, **defaults):
    """
    Returns a `storage` object with the GET and POST arguments.
    See `storify` for how `requireds` and `defaults` work.
    """
    from cStringIO import StringIO
    def dictify(fs): return dict([(k, fs[k]) for k in fs.keys()])

    _method = defaults.pop('_method', 'both')

    e = ctx.env.copy()
    a = b = {}

    if _method.lower() in ['both', 'post']:
        if e['REQUEST_METHOD'] == 'POST':
            a = cgi.FieldStorage(fp = StringIO(data()), environ=e,
              keep_blank_values=1)
            a = dictify(a)

    if _method.lower() in ['both', 'get']:
        e['REQUEST_METHOD'] = 'GET'
        b = dictify(cgi.FieldStorage(environ=e, keep_blank_values=1))

    out = dictadd(b, a)
    try:
        return storify(out, *requireds, **defaults)
    except KeyError:
        badrequest()
        raise StopIteration

def data():
    """Returns the data sent with the request."""
    if 'data' not in ctx:
        cl = intget(ctx.env.get('CONTENT_LENGTH'), 0)
        ctx.data = ctx.env['wsgi.input'].read(cl)
    return ctx.data

def setcookie(name, value, expires="", domain=None):
    """Sets a cookie."""
    if expires < 0:
        expires = -1000000000
    kargs = {'expires': expires, 'path':'/'}
    if domain:
        kargs['domain'] = domain
    # @@ should we limit cookies to a different path?
    cookie = Cookie.SimpleCookie()
    cookie[name] = value
    for key, val in kargs.iteritems():
        cookie[name][key] = val
    header('Set-Cookie', cookie.items()[0][1].OutputString())

def cookies(*requireds, **defaults):
    """
    Returns a `storage` object with all the cookies in it.
    See `storify` for how `requireds` and `defaults` work.
    """
    cookie = Cookie.SimpleCookie()
    cookie.load(ctx.env.get('HTTP_COOKIE', ''))
    try:
        return storify(cookie, *requireds, **defaults)
    except KeyError:
        badrequest()
        raise StopIteration

def debug(*args):
    """
    Prints a prettyprinted version of `args` to stderr.
    """
    try:
        out = ctx.environ['wsgi.errors']
    except:
        out = sys.stderr
    for arg in args:
        print >> out, pprint.pformat(arg)
    return ''

def _debugwrite(x):
    try:
        out = ctx.environ['wsgi.errors']
    except:
        out = sys.stderr
    out.write(x)
debug.write = _debugwrite

class _outputter:
    """Wraps `sys.stdout` so that print statements go into the response."""
    def __init__(self, file): self.file = file
    def write(self, string_):
        if hasattr(ctx, 'output'):
            return output(string_)
        else:
            self.file.write(string_)
    def __getattr__(self, attr): return getattr(self.file, attr)
    def __getitem__(self, item): return self.file[item]

def _capturedstdout():
    sysstd = sys.stdout
    while hasattr(sysstd, 'file'):
        if isinstance(sys.stdout, _outputter): return True
        sysstd = sysstd.file
    if isinstance(sys.stdout, _outputter): return True
    return False

if not _capturedstdout():
    sys.stdout = _outputter(sys.stdout)

_context = {threading.currentThread(): storage()}
ctx = context = threadeddict(_context)

ctx.__doc__ = """
A `storage` object containing various information about the request:

`environ` (aka `env`)
   : A dictionary containing the standard WSGI environment variables.

`host`
   : The domain (`Host` header) requested by the user.

`home`
   : The base path for the application.

`ip`
   : The IP address of the requester.

`method`
   : The HTTP method used.

`path`
   : The path request.

`query`
   : If there are no query arguments, the empty string. Otherwise, a `?` followed
     by the query string.

`fullpath`
   : The full path requested, including query arguments (`== path + query`).

### Response Data

`status` (default: "200 OK")
   : The status code to be used in the response.

`headers`
   : A list of 2-tuples to be used in the response.

`output`
   : A string to be used as the response.
"""

loadhooks = {}
_loadhooks = {}

def load():
    """
    Loads a new context for the thread.

    You can ask for a function to be run at loadtime by
    adding it to the dictionary `loadhooks`.
    """
    _context[threading.currentThread()] = storage()
    ctx.status = '200 OK'
    ctx.headers = []
    if config.get('db_parameters'):
        import db
        db.connect(**config.db_parameters)

    for x in loadhooks.values(): x()

def _load(env):
    load()
    ctx.output = ''
    ctx.environ = ctx.env = env
    ctx.host = env.get('HTTP_HOST')

    #http://groups.google.com/group/webpy/browse_thread/thread/2a4665e54b07c991?pli=1
    if env.get('HTTPS'):
        ctx.protocol = "https"
    else:
        ctx.protocol = "http"
    ctx.homedomain = ctx.protocol + "://" + env.get('HTTP_HOST','[unknown]')
    ctx.homepath = os.environ.get('REAL_SCRIPT_NAME', env.get('SCRIPT_NAME', ''))
    ctx.home = ctx.homedomain + ctx.homepath
    ctx.ip = env.get('REMOTE_ADDR')
    ctx.method = env.get('REQUEST_METHOD')
    ctx.path = env.get('PATH_INFO')
    # http://trac.lighttpd.net/trac/ticket/406 requires:
    if env.get('SERVER_SOFTWARE', '').startswith('lighttpd/'):
        ctx.path = lstrips(env.get('REQUEST_URI').split('?')[0],
                           os.environ.get('REAL_SCRIPT_NAME', env.get('SCRIPT_NAME', '')))

    if env.get('QUERY_STRING'):
        ctx.query = '?' + env.get('QUERY_STRING', '')
    else:
        ctx.query = ''

    ctx.fullpath = ctx.path + ctx.query
    for x in _loadhooks.values(): x()

unloadhooks = {}

def unload():
    """
    Unloads the context for the thread.

    You can ask for a function to be run at loadtime by
    adding it ot the dictionary `unloadhooks`.
    """
    for x in unloadhooks.values(): x()
    # ensures db cursors and such are GCed promptly
    del _context[threading.currentThread()]

def _unload():
    unload()

def wsgifunc(func, *middleware):
    """Returns a WSGI-compatible function from a webpy-function."""
    middleware = list(middleware)

    def wsgifunc(env, start_resp):
        _load(env)
        try:
            result = func()
        except StopIteration:
            result = None
        except:
            print >> debug, traceback.format_exc()
            result = internalerror()

        is_generator = result and hasattr(result, 'next')
        if is_generator:
            # wsgi requires the headers first
            # so we need to do an iteration
            # and save the result for later
            try:
                firstchunk = result.next()
            except StopIteration:
                firstchunk = ''

        status, headers, output = ctx.status, ctx.headers, ctx.output
        ctx._write = start_resp(status, headers)

        # and now, the fun:

        def cleanup():
            # we insert this little generator
            # at the end of our itertools.chain
            # so that it unloads the request
            # when everything else is done

            yield '' # force it to be a generator
            _unload()

        # result is the output of calling the webpy function
        #   it could be a generator...

        if is_generator:
            if firstchunk is flush:
                # oh, it's just our special flush mode
                # ctx._write is set up, so just continue execution
                try:
                    result.next()
                except StopIteration:
                    pass

                _unload()
                return []
            else:
                return itertools.chain([firstchunk], result, cleanup())

        #   ... but it's usually just None
        #
        # output is the stuff in ctx.output
        #   it's usually a string...
        if isinstance(output, str): #@@ other stringlikes?
            _unload()
            return [output]
        #   it could be a generator...
        elif hasattr(output, 'next'):
            return itertools.chain(output, cleanup())
        else:
            _unload()
            raise Exception, "Invalid ctx.output"

    for mw_func in middleware:
        wsgifunc = mw_func(wsgifunc)

    return wsgifunc



# -*- coding: utf-8 -*-
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# FilteredForm contains code copied from django newforms :
# Copyright (c) 2005, the Lawrence Journal-World
#
# Django Licence, see ./newforms_portable/LICENCE
#

from newforms_portable import *
import newforms_portable as newforms
from newforms_portable.forms import BoundField
from newforms_portable.util import ErrorList, escape


import sys, os

import web

#Form
class FilteredForm(newforms.Form):
    """
    used to enable more complex layouts.
    the filter argument contains the names of the fields to render.
    """
    def _html_output_filtered(self, normal_row, error_row, row_ender, help_text_html, errors_on_separate_row, filter):
        """
        Helper function for outputting HTML. Used by as_table(), as_ul(), as_p().
        newforms_plus: 99% c&p from newforms, added filter.
        """
        top_errors = self.non_field_errors() # Errors that should be displayed above all fields.
        output, hidden_fields = [], []
        for name, field in self.fields.items():
            #FilteredForm
            if (filter != None) and (not name in filter):
                continue
            #/FilteredForm
            bf = BoundField(self, field, name)
            bf_errors = ErrorList([escape(error) for error in bf.errors]) # Escape and cache in local variable.
            if bf.is_hidden:
                if bf_errors:
                    top_errors.extend(['(Hidden field %s) %s' % (name, e) for e in bf_errors])
                hidden_fields.append(unicode(bf))
            else:
                if errors_on_separate_row and bf_errors:
                    output.append(error_row % bf_errors)
                label = bf.label and bf.label_tag(escape(bf.label + ':')) or ''
                if field.help_text:
                    help_text = help_text_html % field.help_text
                else:
                    help_text = u''
                output.append(normal_row % {'errors': bf_errors, 'label': label, 'field': unicode(bf), 'help_text': help_text})
        if top_errors:
            output.insert(0, error_row % top_errors)
        if hidden_fields: # Insert any hidden fields in the last row.
            str_hidden = u''.join(hidden_fields)
            if output:
                last_row = output[-1]
                # Chop off the trailing row_ender (e.g. '</td></tr>') and insert the hidden fields.
                output[-1] = last_row[:-len(row_ender)] + str_hidden + row_ender
            else: # If there aren't any rows in the output, just append the hidden fields.
                output.append(str_hidden)
        return u'\n'.join(output)

    def as_table(self , filter = None): #add class="newforms"
        "Returns this form rendered as HTML <tr>s -- excluding the <table></table>."
        return self._html_output_filtered(u'<tr><th class="newforms">%(label)s</th><td class="newforms">%(errors)s%(field)s%(help_text)s</td></tr>', u'<tr><td colspan="2">%s</td></tr>', '</td></tr>', u'<br />%s', False, filter)

    def as_ul(self, filter = None):
        "Returns this form rendered as HTML <li>s -- excluding the <ul></ul>."
        return self._html_output_filtered(u'<li>%(errors)s%(label)s %(field)s%(help_text)s</li>', u'<li>%s</li>', '</li>', u' %s', False , filter)

    def as_p(self , filter = None):
        "Returns this form rendered as HTML <p>s."
        return self._html_output_filtered(u'<p>%(label)s %(field)s%(help_text)s</p>', u'<p>%s</p>', '</p>', u' %s', True, filter)

class Form(FilteredForm):
    info = ""
    title = "No Title"

    def __init__(self,data = None):
        if data == None:
            data = self.initial_data()
        newforms.Form.__init__(self,data)

    def initial_data(self):
        "override in subclass"
        return None

    def start_save(self):
        "called by config_page"
        data = web.Storage(self.cleaned_data)
        self.validate(data)
        self.save(data)
        self.post_save()

    def save(self, vars):
        "override in subclass"
        raise NotImplementedError()

    def post_save(self):
        pass

    def validate(self, data):
        pass

    def pre_html(self):
        return ''

    def post_html(self):
        return ''


#convenience Input Fields.
class CheckBox(newforms.BooleanField):
    "Non Required BooleanField,why the f is it required by default?"
    def __init__(self,label, **kwargs):
        newforms.BooleanField.__init__(self,label=label,required=False,**kwargs)

class IntChoiceField(newforms.ChoiceField):
    """same as ChoiceField, but returns an int
    hint : Use IntChoiceField(choices=enumerate("list","of","strings"]))
             for index-based values on a list of strings.
    """
    def __init__(self, label, choices, **kwargs):
        newforms.ChoiceField.__init__(self, label=label, choices=choices,**kwargs)

    def clean(self, value):
        return int(newforms.ChoiceField.clean(self, value))

class ServerFolder(newforms.CharField):
    def __init__(self, label, **kwargs):
        newforms.CharField.__init__(self, label=label,**kwargs)

    def clean(self, value):
        if value == None:
            value = ""
        value = value.rstrip('/').rstrip('\\')
        self.validate(value)
        return newforms.CharField.clean(self, value)

    def validate(self, value):
        if (value and not os.path.isdir(value)):
            raise newforms.ValidationError(_("This folder does not exist."))

class Password(newforms.CharField):
    def __init__(self, label, **kwargs):
        newforms.CharField.__init__(self, label=label, widget=newforms.PasswordInput,
            **kwargs)

#Lazy multiple select:
class _LazyCheckboxSelectMultiple(newforms.CheckboxSelectMultiple):
    """
    choices are not know at define-time
    choices_getter returns self.choices.
    """
    def __init__(self, attrs=None,choices_getter = None):
        self.choices_getter = choices_getter
        newforms.CheckboxSelectMultiple.__init__(self,attrs)

    def render(self, name, value, attrs=None, choices=()):
        self.choices = self.choices_getter()
        return newforms.CheckboxSelectMultiple.render(self, name, value, attrs, choices)


class LazyMultipleChoice(newforms.MultipleChoiceField):
    """
    choices are not know at define-time
    choices_getter returns self.choices.
    defaults to non-required.
    """
    def __init__(self, label = "",widget=_LazyCheckboxSelectMultiple,
            choices_getter = None, **kwargs):

        self.choices_getter = choices_getter
        #default to non-required
        if not 'required' in kwargs:
            kwargs['required'] = False
        #init, and pass get_choices to the widget.
        newforms.MultipleChoiceField.__init__(self, label=label,
            widget=widget(choices_getter=choices_getter),**kwargs)

    def clean(self, value):
        self.choices = self.choices_getter()
        return newforms.MultipleChoiceField.clean(self, value)

#Deluge specific:
class _DelugeIntInputWidget(newforms.TextInput):
    """
    because deluge-floats are edited as ints.
    """
    def render(self, name, value, attrs=None):
        try:
            value = int(float(value))
            if value == -1 or value == None:
                value = _("Unlimited")
        except:
            pass
        return newforms.TextInput.render(self, name, value, attrs)

class IntegerField(newforms.IntegerField):
    def widget_attrs(self, widget):
        return {'size': "8"}

class FloatField(newforms.FloatField):
    def widget_attrs(self, widget):
        return {'size': "8"}

class DelugeInt(IntegerField):
    def __init__(self, label , **kwargs):
        newforms.IntegerField.__init__(self, label=label, min_value=-1,
            max_value=sys.maxint, widget=_DelugeIntInputWidget, **kwargs)

    def clean(self, value):
        if str(value).lower() == _('Unlimited').lower():
            value = -1
        return int(newforms.IntegerField.clean(self, value))

class DelugeFloat(DelugeInt):
    def clean(self, value):
        try:
            value = int(float(value)) #float->int
        except:
            pass
        return float(DelugeInt.clean(self, value))


class StringList(Field):
    """for a list of strings """
    def __init__(self, label, *args, **kwargs):
        if not "widget" in kwargs:
            kwargs["widget"]  = StringListWidget
        newforms.Field.__init__(self, *args, **kwargs)

    def clean(self, value):
        if type(value) in [str, unicode]: #bug
            return value.split("/n")

class StringListWidget(newforms.Textarea):
    """for a list of strings """
    def __init__(self, attrs=None):
        newforms.Textarea.__init__(self, attrs)

    def render(self, name, value, attrs=None):
        if type(value) in [list, tuple]: #bug
            value = "\n".join(value)
        return newforms.Textarea.render(self, name, value, attrs)

#/fields








#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

#

from deluge import component

menu = component.get("MenuManager")
TB = menu.TOOLBAR_FLAGS

menu.register_admin_page("config", _("Config"), "/config/")
menu.register_admin_page("connect", _("Connect"), "/connect")
menu.register_admin_page("about", _("About"), "/about")
menu.register_admin_page("logout", _("Logout"), "/logout")

menu.register_detail_tab("statistics", _("Statistics"), "tab_statistics")
menu.register_detail_tab("details", _("Details"), "tab_details")
menu.register_detail_tab("options", _("Options"), "tab_options")
menu.register_detail_tab("trackers", _("Trackers"), "tab_trackers")
menu.register_detail_tab("peers", _("Peers"), "tab_peers")
menu.register_detail_tab("files", _("Files"), "tab_files")

menu.register_toolbar_item("add", _("Add"), "list-add.png" , TB.generic, "GET","/torrent/add/", True)
menu.register_toolbar_item("delete",_("Delete"), "list-remove.png" ,TB.torrent_list, "GET","/torrent/delete/" , True)
menu.register_toolbar_item("stop",_("Stop"), "pause.png" ,TB.torrent_list, "POST","/torrent/stop/", True)
menu.register_toolbar_item("start",_("Start"), "start.png" ,TB.torrent_list, "POST","/torrent/start/", True)
menu.register_toolbar_item("queue_up",_("Queue Up"), "queue-up.png" ,TB.torrent_list, "POST","/torrent/queue/up/", True)
menu.register_toolbar_item("queue_down",_("Queue Down"), "queue-down.png" ,TB.torrent_list, "POST","/torrent/queue/down/", True)
menu.register_toolbar_item("queue_top",_("Queue Top"), "go-top.png" ,TB.torrent_list, "POST","/torrent/queue/top/", True)
menu.register_toolbar_item("queue_bottom",_("Queue Bottom"), "go-bottom.png" ,TB.torrent_list, "POST","/torrent/queue/bottom/", True)
menu.register_toolbar_item("details",_("Details"), "details.png" ,TB.torrent, "GET","/torrent/info/", True)
menu.register_toolbar_item("move",_("Move"), "move.png" ,TB.torrent_list,"POST","/torrent/move/", True)
menu.register_toolbar_item("reannounce",_("Reannounce"), "view-refresh.png" ,TB.torrent_list, "POST","/torrent/reannounce/", False)
menu.register_toolbar_item("recheck",_("Recheck"), "edit-redo.png" ,TB.torrent_list, "POST","/torrent/recheck/", False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

#

"""
deluge components.

MenuManager:add torrent-menu-items and torrent-detail tabs.
PageManager: add pages(urls)
PluginManager: deluge plugin manager
ConfigPageManager: add config pages(tabs)

These managers/components are accesible as:

from deluge import component
manager = component.get("ClassName")

"""
from deluge import component
import lib.newforms_plus as forms
from lib.egg_handler import egg_handler
from lib.egg_render import egg_render
from deluge.ui.client import aclient
from deluge import component, pluginmanagerbase
from deluge.configmanager import ConfigManager
from deluge.log import LOG as log

class TOOLBAR_FLAGS:
    generic = 0
    torrent = 1
    torrent_list = 2

class MenuManager(component.Component):
    TOOLBAR_FLAGS = TOOLBAR_FLAGS
    def __init__(self):
        component.Component.__init__(self, "MenuManager")
        self.admin_pages = [] #[(title, url),..]
        self.detail_tabs = [] #[(title, url),..]
        self.toolbar_items = [] #((id,title ,flag ,method ,url ,image ),.. )


        #register vars in template.
        from render import template
        template.Template.globals["admin_pages"] = self.admin_pages
        template.Template.globals["detail_tabs"] = self.detail_tabs
        template.Template.globals["toolbar_items"] = self.toolbar_items


    def register_toolbar_item(self, id, title, image, flag, method, url, important):
        self.toolbar_items.append((id, title, image, flag, method, url, important))

    def deregister_toolbar_item(self, item_id):
        for (i, toolbar) in enumerate(admin_pages):
            if toolbar[0] == item_id:
                del self.toolbar_items[i]

    #admin:
    def register_admin_page(self, id, title, url):
        self.admin_pages.append((id, title, url))

    def deregister_admin_page(page_id):
        for (i, (id, title, url)) in list(enumerate(admin_pages)):
            if id == page_id:
                del self.admin_pages[i]
                return

    #detail:
    def register_detail_tab(self, id, title, page):
        self.detail_tabs.append((id, title, page))

    def deregister_detail_tab(self, tab_id):
        for (i, (id, title, tab)) in list(enumerate(detail_tabs)):
            if id == tab_id:
                del self.detail_tabs[i]
                return

class PageManager(component.Component):
    """
    web,py 0.2 mapping hack..
    see deluge_webserver.py
    """
    def __init__(self):
        component.Component.__init__(self, "PageManager")
        self.page_classes = {}
        self.urls = []
        self.include_javascript = []
        self.ajax_javascript = []

    def register_pages(self, url_list, class_list):
        self.urls += url_list
        self.page_classes.update(class_list)

    def register_page(self, url, klass, use_module=True):
        if use_module:
            name = klass.__module__ + "." + klass.__name__
        else:
            name = klass.__name__
        self.urls.append(url)
        self.urls.append(name)
        self.page_classes[name] = klass

    def deregister_page(self, url):
        raise NotImplemenetedError()
        #self.page_classes[klass.__name__] = None

class PluginManager(pluginmanagerbase.PluginManagerBase,
    component.Component):
    def __init__(self):
        component.Component.__init__(self, "WebPluginManager")
        self.config = ConfigManager("webui06.conf")
        pluginmanagerbase.PluginManagerBase.__init__(
            self, "webui06.conf", "deluge.plugin.webui")

    def start(self):
        """Start the plugin manager"""
        # Update the enabled_plugins from the core
        log.debug("start webui plugin manager")
        aclient.get_enabled_plugins(self._on_get_enabled_plugins)
        aclient.force_call(block=True)

    def stop(self):
        # Disable the plugins
        self.disable_plugins()

    def _on_get_enabled_plugins(self, enabled_plugins):

        log.debug("..Webui has these plugins enabled: %s", enabled_plugins)

        log.debug("abababab")
        log.debug(self.config)

        self.config.config["enabled_plugins"] = enabled_plugins

        # Enable the plugins that are enabled in the config and core
        log.debug(self.enable_plugins)
        try:
            self.enable_plugins()
        except Exception, e:
            log.debug(e)


class ConfigPageManager(component.Component):
    def __init__(self):
        component.Component.__init__(self, "ConfigPageManager")
        self.groups = []
        self.blocks = forms.django.utils.datastructures.SortedDict()

    def register(self, group, name, form):
        if not group in self.groups:
            self.groups.append(group)
        form.group = group
        self.blocks[name] = form

    def deregister(self, name):
        del self.blocks[name]

class PluginApi(component.Component):
    """
    """
    def __init__(self):
        component.Component.__init__(self, "WebPluginApi")
        import web
        from render import render
        import page_decorators as deco
        import utils
        import lib.newforms_plus as forms

        self.egg_handler = egg_handler
        self.egg_render = egg_render
        self.render  = render
        self.web = web
        self.deco = deco
        self.forms = forms
        self.page_manager = component.get("PageManager")
        self.config_page_manager = component.get("ConfigPageManager")
        self.menu_manager = component.get("MenuManager")
        self.utils = utils

def register():
    __page_manager = PageManager()
    __plugin_manager = PluginManager()
    __menu_manager = MenuManager()
    __config_page_manager =  ConfigPageManager()
    __plugin_api =  PluginApi()


if __name__ == "__main__":
    register()






#
# edittrackersdialog.py
#
# Copyright (C) 2007, 2008 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


import gtk, gtk.glade
import pkg_resources

import deluge.common
import deluge.ui.gtkui.common as common
from deluge.ui.client import aclient as client
import deluge.component as component
from deluge.log import LOG as log

class EditTrackersDialog:
    def __init__(self, torrent_id, parent=None):
        self.torrent_id = torrent_id
        self.glade = gtk.glade.XML(
                    pkg_resources.resource_filename("deluge.ui.gtkui",
                                            "glade/edit_trackers.glade"))

        self.dialog = self.glade.get_widget("edit_trackers_dialog")
        self.treeview = self.glade.get_widget("tracker_treeview")
        self.add_tracker_dialog = self.glade.get_widget("add_tracker_dialog")
        self.add_tracker_dialog.set_transient_for(self.dialog)
        self.edit_tracker_entry = self.glade.get_widget("edit_tracker_entry")
        self.edit_tracker_entry.set_transient_for(self.dialog)

        self.dialog.set_icon(common.get_logo(32))

        if parent != None:
            self.dialog.set_transient_for(parent)

        # Connect the signals
        self.glade.signal_autoconnect({
            "on_button_up_clicked": self.on_button_up_clicked,
            "on_button_add_clicked": self.on_button_add_clicked,
            "on_button_edit_clicked": self.on_button_edit_clicked,
            "on_button_edit_cancel_clicked": self.on_button_edit_cancel_clicked,
            "on_button_edit_ok_clicked": self.on_button_edit_ok_clicked,
            "on_button_remove_clicked": self.on_button_remove_clicked,
            "on_button_down_clicked": self.on_button_down_clicked,
            "on_button_ok_clicked": self.on_button_ok_clicked,
            "on_button_cancel_clicked": self.on_button_cancel_clicked,
            "on_button_add_ok_clicked": self.on_button_add_ok_clicked,
            "on_button_add_cancel_clicked": self.on_button_add_cancel_clicked
        })

        # Create a liststore for tier, url
        self.liststore = gtk.ListStore(int, str)

        # Create the columns
        self.treeview.append_column(
            gtk.TreeViewColumn(_("Tier"), gtk.CellRendererText(), text=0))
        self.treeview.append_column(
            gtk.TreeViewColumn(_("Tracker"), gtk.CellRendererText(), text=1))

        self.treeview.set_model(self.liststore)
        self.liststore.set_sort_column_id(0, gtk.SORT_ASCENDING)

    def run(self):
        # Make sure we have a torrent_id.. if not just return
        if self.torrent_id == None:
            return

        # Get the trackers for this torrent

        client.get_torrent_status(
            self._on_get_torrent_status, self.torrent_id, ["trackers"])
        client.force_call()

    def _on_get_torrent_status(self, status):
        """Display trackers dialog"""
        for tracker in status["trackers"]:
            self.add_tracker(tracker["tier"], tracker["url"])

        self.dialog.show()

    def add_tracker(self, tier, url):
        """Adds a tracker to the list"""
        self.liststore.append([tier, url])

    def get_selected(self):
        """Returns the selected tracker"""
        return self.treeview.get_selection().get_selected()[1]

    def on_button_up_clicked(self, widget):
        log.debug("on_button_up_clicked")
        selected = self.get_selected()
        num_rows = self.liststore.iter_n_children(None)
        if selected != None and num_rows > 1:
            tier = self.liststore.get_value(selected, 0)
            new_tier = tier + 1
            # Now change the tier for this tracker
            self.liststore.set_value(selected, 0, new_tier)

    def on_button_add_clicked(self, widget):
        log.debug("on_button_add_clicked")
        # Show the add tracker dialog
        self.add_tracker_dialog.show()
        self.glade.get_widget("textview_trackers").grab_focus()

    def on_button_remove_clicked(self, widget):
        log.debug("on_button_remove_clicked")
        selected = self.get_selected()
        if selected != None:
            self.liststore.remove(selected)

    def on_button_edit_clicked(self, widget):
        """edits an existing tracker"""
        log.debug("on_button_edit_clicked")
        selected = self.get_selected()
        if selected:
            tracker = self.liststore.get_value(selected, 1)
            self.glade.get_widget("entry_edit_tracker").set_text(tracker)
            self.edit_tracker_entry.show()
            self.glade.get_widget("edit_tracker_entry").grab_focus()

    def on_button_edit_cancel_clicked(self, widget):
        log.debug("on_button_edit_cancel_clicked")
        self.edit_tracker_entry.hide()

    def on_button_edit_ok_clicked(self, widget):
        log.debug("on_button_edit_ok_clicked")
        selected = self.get_selected()
        tracker = self.glade.get_widget("entry_edit_tracker").get_text()
        self.liststore.set_value(selected, 1, tracker)
        self.edit_tracker_entry.hide()

    def on_button_down_clicked(self, widget):
        log.debug("on_button_down_clicked")
        selected = self.get_selected()
        num_rows = self.liststore.iter_n_children(None)
        if selected != None and num_rows > 1:
            tier = self.liststore.get_value(selected, 0)
            if not tier > 0:
                return
            new_tier = tier - 1
            # Now change the tier for this tracker
            self.liststore.set_value(selected, 0, new_tier)

    def on_button_ok_clicked(self, widget):
        log.debug("on_button_ok_clicked")
        self.trackers = []
        def each(model, path, iter, data):
            tracker = {}
            tracker["tier"] = model.get_value(iter, 0)
            tracker["url"] = model.get_value(iter, 1)
            self.trackers.append(tracker)
        self.liststore.foreach(each, None)
        # Set the torrens trackers
        client.set_torrent_trackers(self.torrent_id, self.trackers)
        self.dialog.destroy()

    def on_button_cancel_clicked(self, widget):
        log.debug("on_button_cancel_clicked")
        self.dialog.destroy()

    def on_button_add_ok_clicked(self, widget):
        log.debug("on_button_add_ok_clicked")

        # Create a list of trackers from the textview widget
        textview = self.glade.get_widget("textview_trackers")
        trackers = []
        b = textview.get_buffer()
        lines = b.get_text(b.get_start_iter(), b.get_end_iter()).strip().split("\n")
        for l in lines:
            if deluge.common.is_url(l):
                trackers.append(l)

        for tracker in trackers:
            # Figure out what tier number to use.. it's going to be the highest+1
            # Also check for duplicates
            highest_tier = 0
            duplicate = False
            def tier_count(model, path, iter, data):
                tier = model.get_value(iter, 0)
                if tier > data[1]:
                    data[1] = tier
                tracker = model.get_value(iter, 1)
                if data[0] == tracker:
                    # We already have this tracker in the list
                    data[2] = True

            # Check if there are any entries
            if self.liststore.iter_n_children(None) > 0:
                self.liststore.foreach(tier_count, [tracker, highest_tier, duplicate])
            else:
                highest_tier = -1

            # If not a duplicate, then add it to the list
            if not duplicate:
                # Add the tracker to the list
                self.add_tracker(highest_tier + 1, tracker)

        # Clear the entry widget and hide the dialog
        textview.get_buffer().set_text("")
        self.add_tracker_dialog.hide()

    def on_button_add_cancel_clicked(self, widget):
        log.debug("on_button_add_cancel_clicked")
        # Clear the entry widget and hide the dialog
        b = gtk.TextBuffer()
        self.glade.get_widget("textview_trackers").set_buffer(b)
        self.add_tracker_dialog.hide()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) Martijn Voncken 2008 <mvoncken@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

"""
decorators for html-pages.
"""
#relative imports
from render import render
from utils import *
import utils
from deluge.ui.client import sclient as proxy
from deluge.log import LOG as log
#/relative

from web import cookies, setcookie as w_setcookie
from web import url, changequery
from utils import self_url
from render import error_page

#deco's:
def deluge_page_noauth(func):
    """
    add http headers;print result of func
    """
    def deco(self, name = None):
        render.set_global("is_auto_refreshed", False);
        web.header("Content-Type", "text/html; charset=utf-8")
        web.header("Cache-Control", "no-cache, must-revalidate")
        res = func(self, name) #deluge_page_noauth
        print res
    deco.__name__ = func.__name__
    return deco

def check_session(func):
    """
    1:check session
    2:block urls in config.disallow
    return func if session is valid, else redirect to login page.
    mostly used for POST-pages.
    """
    def deco(self, name = None):
        log.debug('%s.%s(name=%s)'  % (self.__class__.__name__, func.__name__,
            name))
        #check disallow config
        current_url = changequery()
        for blocked in utils.config["disallow"]:
            if current_url.startswith(blocked):
                return error_page("Not allowed to : '%s' , Reason: '%s'" %
                    (blocked , utils.config["disallow"][blocked]))
        #/check disallow

        #check session:
        vars = web.input(redir_after_login = None)
        ck = cookies()
        if ck.has_key("session_id") and ck["session_id"] in utils.config["sessions"]:
            return func(self, name) #check_session:ok
        elif vars.redir_after_login:
            utils.seeother(url("/login",redir=self_url()))
        else:
            utils.seeother("/login") #do not continue, and redirect to login page
        #/check session
    deco.__name__ = func.__name__
    return deco

def check_connected(func):
    def deco(self, name = None):
        connected = False
        try:
            proxy.ping()
            connected = True
        except Exception, e:
            log.debug("not_connected: %s" % e)
        if connected:
            return func(self, name) #check_connected:ok
        else:
            utils.seeother("/connect")
    deco.__name__ = func.__name__
    return deco

def deluge_page(func):
    "deluge_page_noauth+check_session+check connected"
    return check_session(check_connected(deluge_page_noauth(func)))

#combi-deco's:
#decorators to use in combination with the ones above.
def torrent_ids(func):
    """
    change page(self, name) to page(self, torrent_ids)
    for pages that allow a list of torrents.
    """
    def deco(self, name):
        return func (self, name.split(',')) #torrent_ids
    deco.__name__ = func.__name__
    return deco

def torrent_list(func):
    """
    change page(self, name) to page(self, torrent_ids)
    for pages that allow a list of torrents.
    """
    def deco(self, name):
        torrent_list = [get_torrent_status(id) for id in name.split(',')]
        return func (self, torrent_list) #torrent_list
    deco.__name__ = func.__name__
    return deco

def torrent(func):
    """
    change page(self, name) to page(self, get_torrent_status(torrent_id))
    """
    def deco(self, name):
        torrent_id = name.split(',')[0]
        torrent =get_torrent_status(torrent_id)
        return func (self, torrent) #torrent
    deco.__name__ = func.__name__
    return deco

def auto_refreshed(func):
    """"
    sets 'is_auto_refreshed' global for templates
    note : decorate AFTER deluge_page_*
    """
    def deco(self, name = None):
        render.set_global("is_auto_refreshed", True);
        return func(self, name) #auto_refreshed
    deco.__name__ = func.__name__
    return deco

def remote(func):
    "decorator for remote (string) api's"
    def deco(self, name = None):
        try:
            log.debug('%s.%s(%s)' ,self.__class__.__name__, func.__name__,name )
            print func(self, name) #remote
        except Exception, e:
            print 'error:%s' % e.message
            print '-'*20
            print  traceback.format_exc()
    deco.__name__ = func.__name__
    return deco



#
# daemon.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#


import deluge.configmanager
import deluge.common
from deluge.log import LOG as log

class Daemon:
    def __init__(self, options, args):
        version = deluge.common.get_version()
        if deluge.common.get_revision() != "":
            version = version + "r" + deluge.common.get_revision()

        log.info("Deluge daemon %s", version)
        log.debug("options: %s", options)
        log.debug("args: %s", args)
        # Set the config directory
        deluge.configmanager.set_config_dir(options.config)

        from deluge.core.core import Core
        # Start the core as a thread and join it until it's done
        self.core = Core(options.port).run()




#
# ui.py
#
# Copyright (C) 2007 Andrew Resch <andrewresch@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#

import deluge.configmanager

from deluge.log import LOG as log

DEFAULT_PREFS = {
    "default_ui": "gtk"
}

class UI:
    def __init__(self, options, args, ui_args):
        log.debug("UI init..")

        # Set the config directory
        deluge.configmanager.set_config_dir(options.config)

        config = deluge.configmanager.ConfigManager("ui.conf", DEFAULT_PREFS)

        if not options.ui:
            selected_ui = config["default_ui"]
        else:
            selected_ui = options.ui

        config.save()
        del config

        try:
            if selected_ui == "gtk":
                log.info("Starting GtkUI..")
                from deluge.ui.gtkui.gtkui import GtkUI
                ui = GtkUI(args)
            elif selected_ui == "web":
                log.info("Starting WebUI..")
                from deluge.ui.webui.webui import WebUI
                ui = WebUI(args)
            elif selected_ui == "console":
                log.info("Starting ConsoleUI..")
                from deluge.ui.console.main import ConsoleUI
                ui = ConsoleUI(ui_args).run()
        except ImportError, e:
            import sys
            import traceback
            error_type, error_value, tb = sys.exc_info()
            stack = traceback.extract_tb(tb)
            last_frame = stack[-1]
            if last_frame[0] == __file__:
                log.error("Unable to find the requested UI: %s.  Please select a different UI with the '-u' option or alternatively use the '-s' option to select a different default UI.", selected_ui)
            else:
                log.exception(e)
                log.error("There was an error whilst launching the request UI: %s", selected_ui)
                log.error("Look at the traceback above for more information.")
            sys.exit(1)
